C> \brief \b ZPSTRF computes the Cholesky factorization with complete
Cpivoting of a complex Hermitian positive semidefinite matrix.
C
C  =========== DOCUMENTATION ===========
C
C Online html documentation available at
C            http://www.netlib.org/lapack/explore-html/
C
C> \htmlonly
C> Download ZPSTRF + dependencies
C> <a
Chref="http://www.netlib.org/cgi-bin/netlibfiles.tgz?format=tgz&filename=/lapack/lapack_routine/zpstrf.f">
C> [TGZ]</a>
C> <a
Chref="http://www.netlib.org/cgi-bin/netlibfiles.zip?format=zip&filename=/lapack/lapack_routine/zpstrf.f">
C> [ZIP]</a>
C> <a
Chref="http://www.netlib.org/cgi-bin/netlibfiles.txt?format=txt&filename=/lapack/lapack_routine/zpstrf.f">
C> [TXT]</a>
C> \endhtmlonly
C
C  Definition:
C  ===========
C
C       SUBROUTINE ZPSTRF( UPLO, N, A, LDA, PIV, RANK, TOL, WORK, INFO )
C
C       .. Scalar Arguments ..
C       DOUBLE PRECISION   TOL
C       INTEGER            INFO, LDA, N, RANK
C       CHARACTER          UPLO
C       ..
C       .. Array Arguments ..
C       COMPLEX*16         A( LDA, * )
C       DOUBLE PRECISION   WORK( 2*N )
C       INTEGER            PIV( N )
C       ..
C
C
C> \par Purpose:
C  =============
C>
C> \verbatim
C>
C> ZPSTRF computes the Cholesky factorization with complete
C> pivoting of a complex Hermitian positive semidefinite matrix A.
C>
C> The factorization has the form
C>    P**T * A * P = U**H * U ,  if UPLO = 'U',
C>    P**T * A * P = L  * L**H,  if UPLO = 'L',
C> where U is an upper triangular matrix and L is lower triangular, and
C> P is stored as vector PIV.
C>
C> This algorithm does not attempt to check that A is positive
C> semidefinite. This version of the algorithm calls level 3 BLAS.
C> \endverbatim
C
C  Arguments:
C  ==========
C
C> \param[in] UPLO
C> \verbatim
C>          UPLO is CHARACTER*1
C>          Specifies whether the upper or lower triangular part of the
C>          symmetric matrix A is stored.
C>          = 'U':  Upper triangular
C>          = 'L':  Lower triangular
C> \endverbatim
C>
C> \param[in] N
C> \verbatim
C>          N is INTEGER
C>          The order of the matrix A.  N >= 0.
C> \endverbatim
C>
C> \param[in,out] A
C> \verbatim
C>          A is COMPLEX*16 array, dimension (LDA,N)
C>          On entry, the symmetric matrix A.  If UPLO = 'U', the
Cleading
C>          n by n upper triangular part of A contains the upper
C>          triangular part of the matrix A, and the strictly lower
C>          triangular part of A is not referenced.  If UPLO = 'L', the
C>          leading n by n lower triangular part of A contains the lower
C>          triangular part of the matrix A, and the strictly upper
C>          triangular part of A is not referenced.
C>
C>          On exit, if INFO = 0, the factor U or L from the Cholesky
C>          factorization as above.
C> \endverbatim
C>
C> \param[in] LDA
C> \verbatim
C>          LDA is INTEGER
C>          The leading dimension of the array A.  LDA >= max(1,N).
C> \endverbatim
C>
C> \param[out] PIV
C> \verbatim
C>          PIV is INTEGER array, dimension (N)
C>          PIV is such that the nonzero entries are P( PIV(K), K ) = 1.
C> \endverbatim
C>
C> \param[out] RANK
C> \verbatim
C>          RANK is INTEGER
C>          The rank of A given by the number of steps the algorithm
C>          completed.
C> \endverbatim
C>
C> \param[in] TOL
C> \verbatim
C>          TOL is DOUBLE PRECISION
C>          User defined tolerance. If TOL < 0, then N*U*MAX( A(K,K) )
C>          will be used. The algorithm terminates at the (K-1)st step
C>          if the pivot <= TOL.
C> \endverbatim
C>
C> \param[out] WORK
C> \verbatim
C>          WORK is DOUBLE PRECISION array, dimension (2*N)
C>          Work space.
C> \endverbatim
C>
C> \param[out] INFO
C> \verbatim
C>          INFO is INTEGER
C>          < 0: If INFO = -K, the K-th argument had an illegal value,
C>          = 0: algorithm completed successfully, and
C>          > 0: the matrix A is either rank deficient with computed
Crank
C>               as returned in RANK, or is not positive semidefinite.
CSee
C>               Section 7 of LAPACK Working Note #161 for further
C>               information.
C> \endverbatim
C
C  Authors:
C  ========
C
C> \author Univ. of Tennessee
C> \author Univ. of California Berkeley
C> \author Univ. of Colorado Denver
C> \author NAG Ltd.
C
C> \date December 2016
C
C> \ingroup complex16OTHERcomputational
C
C  =====================================================================
      SUBROUTINE ZPSTRF( UPLO, N, A, LDA, PIV, RANK, TOL, WORK, INFO )
C
C  -- LAPACK computational routine (version 3.7.0) --
C  -- LAPACK is a software package provided by Univ. of Tennessee,    --
C  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG
C  Ltd..--
C     December 2016
C
C     .. Scalar Arguments ..
      DOUBLE PRECISION   TOL
      INTEGER            INFO, LDA, N, RANK
      CHARACTER          UPLO
C     ..
C     .. Array Arguments ..
      COMPLEX*16         A( LDA, * )
      DOUBLE PRECISION   WORK( 2*N )
      INTEGER            PIV( N )
C     ..
C
C  =====================================================================
C
C     .. Parameters ..
      DOUBLE PRECISION   ONE, ZERO
      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
      COMPLEX*16         CONE
      PARAMETER          ( CONE = ( 1.0D+0, 0.0D+0 ) )
C     ..
C     .. Local Scalars ..
      COMPLEX*16         ZTEMP
      DOUBLE PRECISION   AJJ, DSTOP, DTEMP
      INTEGER            I, ITEMP, J, JB, K, NB, PVT
      LOGICAL            UPPER
C     ..
C     .. External Functions ..
      DOUBLE PRECISION   DLAMCH
      INTEGER            ILAENV
      LOGICAL            LSAME, DISNAN
      EXTERNAL           DLAMCH, ILAENV, LSAME, DISNAN
C     ..
C     .. External Subroutines ..
      EXTERNAL           ZDSCAL, ZGEMV, ZHERK, ZLACGV, ZPSTF2, ZSWAP,
     $                   XERBLA
C     ..
C     .. Intrinsic Functions ..
      INTRINSIC          DBLE, DCONJG, MAX, MIN, SQRT, MAXLOC
C     ..
C     .. Executable Statements ..
C
C     Test the input parameters.
C
      INFO = 0
      UPPER = LSAME( UPLO, 'U' )
      IF( .NOT.UPPER .AND. .NOT.LSAME( UPLO, 'L' ) ) THEN
         INFO = -1
      ELSE IF( N.LT.0 ) THEN
         INFO = -2
      ELSE IF( LDA.LT.MAX( 1, N ) ) THEN
         INFO = -4
      END IF
      IF( INFO.NE.0 ) THEN
         CALL XERBLA( 'ZPSTRF', -INFO )
         RETURN
      END IF
C
C     Quick return if possible
C
      IF( N.EQ.0 )
     $   RETURN
C
C     Get block size
C
      NB = ILAENV( 1, 'ZPOTRF', UPLO, N, -1, -1, -1 )
      IF( NB.LE.1 .OR. NB.GE.N ) THEN
C
C        Use unblocked code
C
         CALL ZPSTF2( UPLO, N, A( 1, 1 ), LDA, PIV, RANK, TOL, WORK,
     $                INFO )
         GO TO 230
C
      ELSE
C
C     Initialize PIV
C
         DO 100 I = 1, N
            PIV( I ) = I
  100    CONTINUE
C
C     Compute stopping value
C
         DO 110 I = 1, N
            WORK( I ) = DBLE( A( I, I ) )
  110    CONTINUE
         PVT = MAXLOC( WORK( 1:N ), 1 )
         AJJ = DBLE( A( PVT, PVT ) )
         IF( AJJ.LE.ZERO.OR.DISNAN( AJJ ) ) THEN
            RANK = 0
            INFO = 1
            GO TO 230
         END IF
C
C     Compute stopping value if not supplied
C
         IF( TOL.LT.ZERO ) THEN
            DSTOP = N * DLAMCH( 'Epsilon' ) * AJJ
         ELSE
            DSTOP = TOL
         END IF
C
C
         IF( UPPER ) THEN
C
C           Compute the Cholesky factorization P**T * A * P = U**H * U
C
            DO 160 K = 1, N, NB
C
C              Account for last block not being NB wide
C
               JB = MIN( NB, N-K+1 )
C
C              Set relevant part of first half of WORK to zero,
C              holds dot products
C
               DO 120 I = K, N
                  WORK( I ) = 0
  120          CONTINUE
C
               DO 150 J = K, K + JB - 1
C
C              Find pivot, test for exit, else swap rows and columns
C              Update dot products, compute possible pivots which are
C              stored in the second half of WORK
C
                  DO 130 I = J, N
C
                     IF( J.GT.K ) THEN
                        WORK( I ) = WORK( I ) +
     $                              DBLE( DCONJG( A( J-1, I ) )*
     $                                    A( J-1, I ) )
                     END IF
                     WORK( N+I ) = DBLE( A( I, I ) ) - WORK( I )
C
  130             CONTINUE
C
                  IF( J.GT.1 ) THEN
                     ITEMP = MAXLOC( WORK( (N+J):(2*N) ), 1 )
                     PVT = ITEMP + J - 1
                     AJJ = WORK( N+PVT )
                     IF( AJJ.LE.DSTOP.OR.DISNAN( AJJ ) ) THEN
                        A( J, J ) = AJJ
                        GO TO 220
                     END IF
                  END IF
C
                  IF( J.NE.PVT ) THEN
C
C                    Pivot OK, so can now swap pivot rows and columns
C
                     A( PVT, PVT ) = A( J, J )
                     CALL ZSWAP( J-1, A( 1, J ), 1, A( 1, PVT ), 1 )
                     IF( PVT.LT.N )
     $                  CALL ZSWAP( N-PVT, A( J, PVT+1 ), LDA,
     $                              A( PVT, PVT+1 ), LDA )
                     DO 140 I = J + 1, PVT - 1
                        ZTEMP = DCONJG( A( J, I ) )
                        A( J, I ) = DCONJG( A( I, PVT ) )
                        A( I, PVT ) = ZTEMP
  140                CONTINUE
                     A( J, PVT ) = DCONJG( A( J, PVT ) )
C
C                    Swap dot products and PIV
C
                     DTEMP = WORK( J )
                     WORK( J ) = WORK( PVT )
                     WORK( PVT ) = DTEMP
                     ITEMP = PIV( PVT )
                     PIV( PVT ) = PIV( J )
                     PIV( J ) = ITEMP
                  END IF
C
                  AJJ = SQRT( AJJ )
                  A( J, J ) = AJJ
C
C                 Compute elements J+1:N of row J.
C
                  IF( J.LT.N ) THEN
                     CALL ZLACGV( J-1, A( 1, J ), 1 )
                     CALL ZGEMV( 'Trans', J-K, N-J, -CONE, A( K, J+1 ),
     $                           LDA, A( K, J ), 1, CONE, A( J, J+1 ),
     $                           LDA )
                     CALL ZLACGV( J-1, A( 1, J ), 1 )
                     CALL ZDSCAL( N-J, ONE / AJJ, A( J, J+1 ), LDA )
                  END IF
C
  150          CONTINUE
C
C              Update trailing matrix, J already incremented
C
               IF( K+JB.LE.N ) THEN
                  CALL ZHERK( 'Upper', 'Conj Trans', N-J+1, JB, -ONE,
     $                        A( K, J ), LDA, ONE, A( J, J ), LDA )
               END IF
C
  160       CONTINUE
C
         ELSE
C
C        Compute the Cholesky factorization P**T * A * P = L * L**H
C
            DO 210 K = 1, N, NB
C
C              Account for last block not being NB wide
C
               JB = MIN( NB, N-K+1 )
C
C              Set relevant part of first half of WORK to zero,
C              holds dot products
C
               DO 170 I = K, N
                  WORK( I ) = 0
  170          CONTINUE
C
               DO 200 J = K, K + JB - 1
C
C              Find pivot, test for exit, else swap rows and columns
C              Update dot products, compute possible pivots which are
C              stored in the second half of WORK
C
                  DO 180 I = J, N
C
                     IF( J.GT.K ) THEN
                        WORK( I ) = WORK( I ) +
     $                              DBLE( DCONJG( A( I, J-1 ) )*
     $                                    A( I, J-1 ) )
                     END IF
                     WORK( N+I ) = DBLE( A( I, I ) ) - WORK( I )
C
  180             CONTINUE
C
                  IF( J.GT.1 ) THEN
                     ITEMP = MAXLOC( WORK( (N+J):(2*N) ), 1 )
                     PVT = ITEMP + J - 1
                     AJJ = WORK( N+PVT )
                     IF( AJJ.LE.DSTOP.OR.DISNAN( AJJ ) ) THEN
                        A( J, J ) = AJJ
                        GO TO 220
                     END IF
                  END IF
C
                  IF( J.NE.PVT ) THEN
C
C                    Pivot OK, so can now swap pivot rows and columns
C
                     A( PVT, PVT ) = A( J, J )
                     CALL ZSWAP( J-1, A( J, 1 ), LDA, A( PVT, 1 ), LDA )
                     IF( PVT.LT.N )
     $                  CALL ZSWAP( N-PVT, A( PVT+1, J ), 1,
     $                              A( PVT+1, PVT ), 1 )
                     DO 190 I = J + 1, PVT - 1
                        ZTEMP = DCONJG( A( I, J ) )
                        A( I, J ) = DCONJG( A( PVT, I ) )
                        A( PVT, I ) = ZTEMP
  190                CONTINUE
                     A( PVT, J ) = DCONJG( A( PVT, J ) )
C
C
C                    Swap dot products and PIV
C
                     DTEMP = WORK( J )
                     WORK( J ) = WORK( PVT )
                     WORK( PVT ) = DTEMP
                     ITEMP = PIV( PVT )
                     PIV( PVT ) = PIV( J )
                     PIV( J ) = ITEMP
                  END IF
C
                  AJJ = SQRT( AJJ )
                  A( J, J ) = AJJ
C
C                 Compute elements J+1:N of column J.
C
                  IF( J.LT.N ) THEN
                     CALL ZLACGV( J-1, A( J, 1 ), LDA )
                     CALL ZGEMV( 'No Trans', N-J, J-K, -CONE,
     $                           A( J+1, K ), LDA, A( J, K ), LDA, CONE,
     $                           A( J+1, J ), 1 )
                     CALL ZLACGV( J-1, A( J, 1 ), LDA )
                     CALL ZDSCAL( N-J, ONE / AJJ, A( J+1, J ), 1 )
                  END IF
C
  200          CONTINUE
C
C              Update trailing matrix, J already incremented
C
               IF( K+JB.LE.N ) THEN
                  CALL ZHERK( 'Lower', 'No Trans', N-J+1, JB, -ONE,
     $                        A( J, K ), LDA, ONE, A( J, J ), LDA )
               END IF
C
  210       CONTINUE
C
         END IF
      END IF
C
C     Ran to completion, A has full rank
C
      RANK = N
C
      GO TO 230
  220 CONTINUE
C
C     Rank is the number of steps completed.  Set INFO = 1 to signal
C     that the factorization cannot be used to solve a system.
C
      RANK = J - 1
      INFO = 1
C
  230 CONTINUE
      RETURN
C
C     End of ZPSTRF
C
      END

