      SUBROUTINE DIPOLES
C
C     Purpose: Calculation of dipole integral matrix.
C
C     History: - Creation (21.12.05, AMK)
C                         (05.04.06, RFM) 
C                         (24.03.11, GG)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DCOP: Dimension of Cartesian orbitals.
C     DSHL: Dimension of shell blocks.
C     DSPP: Dimension of scaling parameters.
C
C     List of local variables:
C
C     L{A/B}   : Orbital L quantum number.
C     NGTO{A/B}: Number of contractions.
C     SKIPSHL  : Skipping of shell combination, if true.
C
C     List of local dynamical fields:
C
C     HRRINT: Work field for horizontal recurrence relation.
C     MOMBLK: Diatomic moment integral shell block.
C     MOMMAT: Dipole moment operator matrix.
C     SHLBLK: Diatomic integral shell block.
C     VRRINT: Work field for vertical recurrence relation.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE 
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'pointer.h'
      INCLUDE 'strings.h'
C
      LOGICAL SKIPSHL
      CHARACTER*1 LTU
      INTEGER IATOM,ISHL,IXYZ,JATOM,JSHL,LA,LB,LC,NGTOA,NGTOB
C
      INTEGER ALLOCATION,DCOP,DIMCOP,DIMSOP,DSHL,DSPP
      REAL,ALLOCATABLE :: HRRINT(:,:),MOMBLK(:,:,:),MOMMAT(:,:),
     $                    SHLBLK(:,:),VRRINT(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      DCOP = DIMCOP(2*LBASMAX(0)+1)
      DSHL = DIMSOP(LBASMAX(0)+1)
      DSPP = DIMSOP(2*LBASMAX(0)+1)
C
      ALLOCATE(HRRINT(DCOP,DCOP),MOMBLK(DSHL,DSHL,4),MOMMAT(NSTO,NSTO),
     $         SHLBLK(DSHL,DSHL),VRRINT(DCOP,DSPP),STAT=ALLOCATION)
CCC ADLL
      IF (ALLOCATION.GT.0) CALL ERRMSG('DIPOLES','ALLOCATION FAILED',1)
C      CALL CHKERRSTAT(ALLOCATION,'DIPOLES','ALLOCATION FAILED')
CCC ADLL
C
C     *** Initialize integral recurrence fields ***
C
      VRRINT(:,:) = 0.0
C
C     *** Loop over X/Y/Z components ***
C
      DO 60 IXYZ=1,3
C
C     *** Initialize dipole matrix ***
C
        MOMMAT(:,:) = 0.0
C
C     *** Calculate dipole matrix ***
C
        DO 50 IATOM=1,NATOM
          DO 40 JATOM=IATOM,NATOM
            CALL ATOMDIS(IATOM,JATOM,RAB,RSQ)
            CALL ORGDIS(IATOM,JATOM,RAC,RBC)
            DO 30 ISHL=LLSHL(IATOM),ULSHL(IATOM)
              DO 20 JSHL=MAX(ISHL,LLSHL(JATOM)),ULSHL(JATOM)
                LA = SHLPTR(ISHL,3)
                LB = SHLPTR(JSHL,3)
                CALL OSETUP(LA,LB,NGTOA,NGTOB,ISHL,JSHL)
                CALL GTOPRO(NGTOA,NGTOB,SKIPSHL)
                IF (SKIPSHL) GO TO 20
                CALL OCONT(LA,LB,1,NGTOA,NGTOB,0,VRRINT,DCOP,DSPP)
                CALL OVRR(LA,LB,1,HRRINT,VRRINT,DCOP,DSPP)
C
C     *** Build dipole integrals from basic overlap integrals ***
C
                MOMBLK(:,:,:) = 0.0
                DO 10 LC=0,1
                  CALL MHRR(LA,LB,LC,HRRINT,SHLBLK,DCOP,DSHL)
                  CALL MSRR(LA,LB,LC,1,MOMBLK,SHLBLK,4,DSHL)
   10           CONTINUE
C
C     *** Transpose MOMBLK, if LA < LB ***
C
                IF (LA.LT.LB) THEN
                  MOMBLK(:,:,IXYZ+1) = TRANSPOSE(MOMBLK(:,:,IXYZ+1))
                END IF
C
C     *** Build dipole integral matrices ***
C
                CALL BLDIMAT(LA,LB,ISHL,JSHL,LLSTO,NCSTO,MOMMAT,
     $                       MOMBLK(:,:,IXYZ+1),NSTO,DSHL)
C
   20         CONTINUE
   30       CONTINUE
   40     CONTINUE
   50   CONTINUE
C
C     *** Symmetrize MOMMAT ***
C
        CALL SYMMAT(MOMMAT,NSTO,NSTO,'UPLOW')
C
C     *** Store dipole integral matrix ***
C
        IF (MPMASTER.OR.(STATUS.NE.'MAGNETIC')) THEN
          CALL FFPIO(MOMMAT,NSTO,'WRITE',LTU(XYZ(IXYZ)))
        END IF
C
   60 CONTINUE
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(HRRINT,MOMBLK,MOMMAT,SHLBLK,VRRINT,STAT=ALLOCATION)
CCC ADLL
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('DIPOLES','DEALLOCATION FAILED',1)
      END IF
C      CALL CHKERRSTAT(ALLOCATION,'DIPOLES','DEALLOCATION FAILED')
CCC ADLL
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE DIPOLES ***
C
      END
