      SUBROUTINE COULOMB(KSA,KSB,XCVA,XCVB,NCDCA,NCDCB,DSTO,DSTO_ME)
C
C     Purpose: Calculate Coulomb contribution of the Kohn-Sham matrix.
C
C     Lit.: L.E. McMurchie, E.R. Davidson, J. Comp. Phys. 26, 218 (1978)
C           A.M. Koster, J. Chem. Phys. 118, 9943 (2003)
C           A. Alvarez, A.M. Koster, J. Chem. Phys. 139, 024102 (2013)
C
C     History: - Creation (13.02.15, AMK)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO   : Dimension of Slater typ orbital matrix.
C     DSTO_ME: Dimension of Slater typ orbital matrix on node ME.
C
C     List of local variables:
C
C     KSA  : Alpha Kohn-Sham matrix.
C     KSB  : Beta Kohn-Sham matrix.
C     NCDCA: Alpha charge density difference coefficients.
C     NCDCB: Beta charge density difference coefficients.
C     QCDCA: Alpha charge density difference moments.
C     QCDCB: Beta charge density difference moments.
C     XCVA : Alpha exchange-correlation potential matrix.
C     XCVB : Beta exchange-correlation potential matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'iteration.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'opt.h'
C
      REAL,PARAMETER :: DUMMY(1) = (/999.999/)
C
      INTEGER DSTO,DSTO_ME
      REAL KSA(DSTO,DSTO),KSB(DSTO,DSTO)
      REAL XCVA(DSTO_ME,DSTO_ME),XCVB(DSTO_ME,DSTO_ME)
C
      REAL NCDCA(MAXAUX),NCDCB(MAXAUX)
C
      LOGICAL STATED
C
C     ------------------------------------------------------------------
C
C     *** Synchronize parallel processes ***
C
      CALL MP_SYNC
C
C     *** Suspend external timers for ERI calculation ***
C
      IF (STATUS.EQ.'FFP') THEN
        CALL TIMER('SUSPEND','FFP ITERATION')
      ELSE IF (STATUS.EQ.'FREQ') THEN
        CALL TIMER('SUSPEND','FORCE CONSTANTS')
      ELSE IF ((STATUS.EQ.'HESSIAN').OR.(STATUS.EQ.'UPDATE')) THEN
        IF (SADDLE) THEN
          CALL TIMER('SUSPEND','SADDLE INTERPOLATION')
        ELSE
          CALL TIMER('SUSPEND','OPTIMIZATION CYCLE')
        END IF
      ELSE IF (.NOT.STATED('END')) THEN
CCC ADLL
C        CALL TIMER('SUSPEND','SCF ITERATION')
      ELSE IF (STATUS.EQ.'RTDFT') THEN
        CALL TIMER('SUSPEND','ELECTRON DYNAMICS SIMULATION')
CCC ADLL
      END IF
C
C     *** Load previous Hartree matrix ***
C
      IF (MPMASTER) THEN
        CALL SCFIO(KSA,DSTO,'READ','KSALPHA')
        IF (OKS) CALL SCFIO(KSB,DSTO,'READ','KSBETA')
      ELSE
        KSA(:,:) = 0.0
        IF (OKS) KSB(:,:) = 0.0
      END IF
C
C     *** Calculate mixed SCF ERIs and add to Hartree matrix ***
C
      IF (ERIRAM) THEN
C
C     *** Load near-field ERIs and add to Hartree matrix ***
C
        CALL RAMERIS(KSA,KSB,NCDCA,NCDCB,DSTO)
C
C     *** Build Hermite charge density coefficients and moments ***
C
        IF (MPMASTER) THEN
          CALL BLDCDC(NCDCA,'HERMITE')
          IF (OKS) CALL BLDCDC(NCDCB,'HERMITE')
          CALL BLDCDM(NCDCA,QCDCA,'BUILD')
          IF (OKS) CALL BLDCDM(NCDCB,QCDCB,'BUILD')
        END IF
C
C     *** Calculate far-field ERIs and add to Hartree matrix ***
C
        CALL FARERIS(KSA,KSB,QCDCA,QCDCB,DSTO)
C
C     *** Calculate direct SCF ERIs and add to Hartree matrix ***
C
      ELSE
C
C     *** Build Hermite charge density coefficients and moments ***
C
        IF (MPMASTER) THEN
          CALL BLDCDC(NCDCA,'HERMITE')
          IF (OKS) CALL BLDCDC(NCDCB,'HERMITE')
          CALL BLDCDM(NCDCA,QCDCA,'BUILD')
          IF (OKS) CALL BLDCDM(NCDCB,QCDCB,'BUILD')
        END IF
C
C     *** Calculate near-field ERIs and add to Hartree matrix ***
C
        CALL NEARERIS(KSA,KSB,NCDCA,NCDCB,DSTO)
C
C     *** Calculate far-field ERIs and add to Hartree matrix ***
C
        CALL FARERIS(KSA,KSB,QCDCA,QCDCB,DSTO)
C
      END IF
C
C     *** Collecting of the results on the master node ***
C
      CALL MP_COLLECT(KSA,DSTO*NSTO,MASTER,DUMMY,.FALSE.)
      IF (OKS) CALL MP_COLLECT(KSB,DSTO*NSTO,MASTER,DUMMY,.FALSE.)
C
C     *** Symmetrize and save Hartree matrices ***
C
      IF (MPMASTER) THEN
C
        CALL SYMMAT(KSA,DSTO,NSTO,'UPLOW')
        IF (OKS) CALL SYMMAT(KSB,DSTO,NSTO,'UPLOW')
C
        CALL SCFIO(KSA,DSTO,'WRITE','KSALPHA')
        IF (OKS) CALL SCFIO(KSB,DSTO,'WRITE','KSBETA')
C
C     *** Add exchange correlation potential ***
C
        IF (EXX.OR.(.NOT.CDXC)) THEN
          KSA(:,:) = KSA(:,:) + XCVA(:,:)
          IF (OKS) KSB(:,:) = KSB(:,:) + XCVB(:,:)
        END IF
C
        IF (ROKS.AND.CONVERGED) THEN
          IF (.NOT.STATED('ATOM')) THEN
            CALL SCFIO(KSA,NSTO,'WRITE','ROALPHA')
            CALL SCFIO(KSB,NSTO,'WRITE','ROBETA')
          END IF
        END IF
C
C     *** Scale open shell Kohn-Sham matrices ***
C
        IF (OKS) THEN
          CALL SCALMAT(KSA,DSTO,NSTO,NSTO,'TOTAL',0.5)
          CALL SCALMAT(KSB,DSTO,NSTO,NSTO,'TOTAL',0.5)
        END IF
C
      END IF
C
C     *** Continue external timers after ERI calculation ***
C
      IF (STATUS.EQ.'FFP') THEN
        CALL TIMER('CONTINUE','FFP ITERATION')
      ELSE IF (STATUS.EQ.'FREQ') THEN
        CALL TIMER('CONTINUE','FORCE CONSTANTS')
      ELSE IF ((STATUS.EQ.'HESSIAN').OR.(STATUS.EQ.'UPDATE')) THEN
        IF (SADDLE) THEN
          CALL TIMER('CONTINUE','SADDLE INTERPOLATION')
        ELSE
          CALL TIMER('CONTINUE','OPTIMIZATION CYCLE')
        END IF
      ELSE IF (.NOT.STATED('END')) THEN
CCC ADLL
C        CALL TIMER('CONTINUE','SCF ITERATION')
      ELSE IF (STATUS.EQ.'RTDFT') THEN
        CALL TIMER('CONTINUE','ELECTRON DYNAMICS SIMULATION')
CCC ADLL
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE COULOMB ***
C
      END
