      SUBROUTINE MODXPOT(RHOMOD,DRHOMOD,VXSPIN,VXGSPIN,PNTMSK,NGPB)
C
C     Purpose: Calculate modified exchange potential.
C
C     History: - Creation (04.03.08, JCE)
C                         (01.11.11, AMK)
C
C     ******************************************************************
C
C     List of local variables:
C
C     DRHOMOD: Norm of the gradient of the modified density.
C     NGPB   : Number of grid points in the current batch.
C     PNTMSK : Grid point screening mask.
C     RHOMOD : Modified density.
C     VXSPIN : Modified alpha or beta exchange potential.
C     VXGSPIN: Modified GGA correction to the exchange potential.
C
C     List of exchange potential subroutines:
C
C     VXALPHA : X-alpha (Dirac) exchange potential.
C     VXLSD   : Uniform electron gas exchange potential.
C     VXPW86  : Perdew-Wang GGA exchange potential (1986).
C     VXB88   : Becke GGA exchange potential (1988).
C     VXPW91  : Perdew-Wang GGA exchange potential (1991).
C     VXECMV  : Engel et al. GGA exchange potential (1992).
C     VXEV93  : Engel & Vosko GGA exchange potential (1993).
C     VXLB94  : Leeuwen-Baereends model potential (1994).
C     VXPBE96 : Perdew, Burke & Ernzerhof GGA exchange potential (1996).
C     VXPBE98 : Zhang & Yang PBE exchange potential (1998).
C     VXPPB99 : Hammer, Hansen & Norskov PBE exchange potential (1999).
C     VXPBESOL: PBEsol GGA exchange potential (2008).
C     VXHC    : Handy & Cohen GGA exchange potential (2001).
C     VXKT1   : Keal & Tozer GGA exchange potential 1 (2003).
C     VXKT2   : Keal & Tozer GGA exchange potential 2 (2003).
C     VXKT3   : Keal & Tozer GGA exchange potential 3 (2004).
C     VXVMT   : Vela, Medel & Trickey GGA exchange potential (2009).
C     VXVMTSOL: VMTsol exchange potential (2009).
C     VXSO11  : Peverati et al. GGA exchange potential (2011).
C     VXN12   : Peverati & Truhlar GGA exchange potential (2012).
C     VXCAP   : Carmona-Espindola et al. GGA exchange potential (2014).
C     VXGAM   : Yu et al. GGA type exchange potential (2015).
C     VXB3LYP : B3LYP hybrid exchange potential DFT part (1994).
C     VXPBE0  : PBE0 hybrid exchange potential DFT part (1996).
C     VXCAMB88: CAMB3LYP RS hybrid exchange potential DFT part (2004).
C     VXCAMPBE: CAMPBE0 RS hybrid exchange potential DFT part (2004).
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'flags.h'
C
      INTEGER NGPB
      LOGICAL PNTMSK(NGPB)
      REAL RHOMOD(NGPB),DRHOMOD(NGPB)
      REAL VXSPIN(NGPB),VXGSPIN(NGPB)
C
      INTEGER IGP
C
C     ------------------------------------------------------------------
C
C     *** Calculate exchange potential with modified density ***
C
      IF (XTYP(1:6).EQ.'XALPHA') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXALPHA(RHOMOD(IGP),VXSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'LSD') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXLSD(RHOMOD(IGP),VXSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PW86') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXPW86(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                  VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'B88') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXB88(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                 VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PW91') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXPW91(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                  VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'ECMV') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXECMV(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                  VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'EV93') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXEV93(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                  VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'LB94') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXLB94(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                  VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'PBE96') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXPBE96(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                   VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'PBE98') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXPBE98(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                   VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'PBE99') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXPBE99(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                   VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'PBESOL') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXPBESOL(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                    VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'OPTX') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXHC(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'KT1X') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXKT1(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                 VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'KT2X') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXKT2(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                 VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'KT3X') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXKT3(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                 VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'VMT09') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXVMT(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                 VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'VMTSOL') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXVMTSOL(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                    VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'SO11') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXSO11(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                  VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'N12') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXN12(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                 VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'CAP') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXCAP(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                 VXGSPIN(IGP))
          END IF
        END DO
C
C
cej
      ELSE IF (XTYP(1:5).EQ.'XNCAP') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXNCAP(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                  VXGSPIN(IGP))
          END IF
        END DO
C

      ELSE IF (XTYP(1:3).EQ.'GAM') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXGAM(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                 VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'B3LYP') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXB3LYP(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                   VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PBE0') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXPBE0(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                  VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'CAMB88') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXCAMB88(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                    VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'CAMPBE') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXCAMPBE(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                    VXGSPIN(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'HSE') THEN
C
        DO IGP=1,NGPB
          IF (PNTMSK(IGP)) THEN
            CALL VXHSE(RHOMOD(IGP),DRHOMOD(IGP),VXSPIN(IGP),
     $                 VXGSPIN(IGP))
          END IF
        END DO
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE MODXPOT ***
C
      END
