      SUBROUTINE NUMPOPDIR_SCREENED(PA,PB,DSTO)
C
C     Purpose: Numerical calculation of atom populations by weighted
C              density integration. The grid is retrieved (directly) 
C              on the fly.
C
C     Lit.: G. Geudtner, et al., J. Comp. Chem. 27, 483 (2006)
C
C     History: - Creation (09.15.11, ADLL)
C                         (30.08.12, AMK)
C                         (06.11.19, ADLL)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DRAD: Dimension of radial shells on the grid.
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     LLATOM : Lower limit of atoms in current buffer (record).
C     LLBATCH: Lower limit of a grid point batch.
C     LLGPB  : Lower limit grid point of a batch.
C     NRAD   : Number of radial shells for the full grid.
C     PA     : Alpha density matrix / Alpha potential matrix.
C     PB     : Beta density matrix / Beta potential matrix.
C     ULATOM : Upper limit of atoms in current buffer (record).
C     ULBATCH: Upper limit of a grid point batch.
C     ULGPB  : Upper limit grid point of a batch.
C     
C     List of local dynamical fields:
C
C     HTO   : Function values of Hermite typ orbitals.
C     RADGRD: Radial PoinTs PoinTeR RADGRD(IRAD,N).
C             N = 1: Abscissa of IRAD.
C             N = 2: Radial weight of IRAD.
C     RADPTR: RADial PoinTeR RADPTR(IRAD,N).
C             N = 1: Atom of the current radial shell IRAD.
C             N = 2: Radial shell in the field NRADIAL.
C     STO   : Function values of the basis functions.
C     WATOM : Atomic weights for density distribution.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'xc.h'
C
      INTEGER DSTO
      REAL PA(DSTO,DSTO),PB(DSTO,DSTO)
C
      LOGICAL DONE
      INTEGER IATOM,IBATCH,IRAD,JATOM,NGPB,NRAD
      INTEGER LLATOM,LLBATCH,LLGPB,LL_ME,ULATOM,ULBATCH,ULGPB,UL_ME
C
      INTEGER ALLOCATION,DRAD,NSPIN
      INTEGER,ALLOCATABLE :: RADPTR(:,:)
      REAL,ALLOCATABLE :: HTO(:,:),RADGRD(:,:),STO(:,:),WATOM(:,:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields for numerical integration ***
C
      NSPIN = 1
      IF (ITHFO) NSPIN = 2
C
      DRAD = MAXRAD*NATOM
      ALLOCATE(RADGRD(DRAD,2),RADPTR(DRAD,2),STAT=ALLOCATION)
C
      IF (TOPCD) THEN
        ALLOCATE(HTO(MAXGPB,NALL),WATOM(MAXGPB,NATOM,NSPIN),
     $           STAT=ALLOCATION)
      ELSE IF (HPA) THEN
        ALLOCATE(HTO(MAXGPB,NALL),STO(MAXGPB,DSTO),
     $           WATOM(MAXGPB,NATOM,NSPIN),STAT=ALLOCATION)
      ELSE
        ALLOCATE(STO(MAXGPB,DSTO),WATOM(MAXGPB,NATOM,NSPIN),
     $           STAT=ALLOCATION)
      END IF 
C
      CALL CHKERRSTAT(ALLOCATION,'NUMPOPDIR','ALLOCATION FAILED')
C
C     *** Initialize radial pointer, grid rotation and loop loads ***
C
      CALL RADINI(IATOM,NRAD,RADGRD,RADPTR,DRAD,LL_ME,UL_ME,MP_AIMS)
C
      IF (ROTGRD.AND.(LL_ME.GE.1)) THEN
        CALL GRIDROT(IATOM,RADGRD,RADPTR,DRAD,LL_ME,'STORE')
        CALL GRIDROT(IATOM,RADGRD,RADPTR,DRAD,LL_ME,'INITME')
      END IF
C
C     *** Loop over all radial shells ***
C
      DONE = .FALSE.
C
      DO IRAD=LL_ME,UL_ME
C
C     *** If necessary, update atom and radial pointer ***
C
        CALL RADUPD(IATOM,IRAD,RADGRD,RADPTR,DRAD,LL_ME,DONE,.TRUE.)
C
C     *** Radial sphere generation ***
C
        CALL RADGEN(IATOM,IRAD,RADGRD,RADPTR,DRAD,LL_ME,UL_ME,.TRUE.)
C
C     *** Do the numerical integration if buffer is filled ***
C
        IF (((NGPBUF+NGPSPH).GT.MAXGP).OR.(IRAD.EQ.UL_ME)) THEN
C
C     *** Set upper grid point limit for current atom ***
C
          ULGP(IATOM) = NGPBUF
C  
C     *** Save number of grid points for the last atom ***
C
          IF (ROTGRD.AND.(IRAD.EQ.UL_ME)) THEN
            CALL GRIDROT(IATOM,RADGRD,RADPTR,DRAD,LL_ME,'RESTORE')
          END IF
          NGPVEC(IATOM) = NGPATOM
C
C     *** Define lower and upper limits of atoms in buffer ***
C
          LLATOM = IATOM - NATOMBUF + 1
          ULATOM = LLATOM + NATOMBUF - 1
C
C     *** Loop over all atoms in current grid vector buffer ***
C
          DO JATOM=LLATOM,ULATOM
C
C     *** Find neighbors for JATOM ***
C
            CALL NEIGHBOR(JATOM)
C
C     *** Define lower and upper limits of ***
C     *** the current batch of grid points ***
C
            LLBATCH = (LLGP(JATOM) - 1)/MAXGPB + 1
            ULBATCH = (ULGP(JATOM) - 1)/MAXGPB + 1
C
C     *** Loop over all grid point batches of atom JATOM ***
C
            DO IBATCH=LLBATCH,ULBATCH
C
              IF (IBATCH.EQ.LLBATCH) THEN
                LLGPB = LLGP(JATOM)
                ULGPB = MIN(ULGP(JATOM),IBATCH*MAXGPB)
              ELSE IF (IBATCH.EQ.ULBATCH) THEN
                LLGPB = (IBATCH - 1)*MAXGPB + 1
                ULGPB = ULGP(JATOM)
              ELSE
                ULGPB = IBATCH*MAXGPB
                LLGPB = ULGPB - MAXGPB + 1
              END IF
C
              NGPB = ULGPB - LLGPB + 1
C
C     *** Get chi or tau values and build density ***
C
              IF (HPA) THEN
                CALL TAU(HTO,NALL,LLGPB,NGPB)
                CALL CHI(STO,DSTO,LLGPB,NGPB)
                CALL RHO_SCREENED(PA,STO,DSTO,RHOA,LLGPB,NGPB)
                IF (OKS) CALL RHO_SCREENED(PB,STO,DSTO,RHOB,LLGPB,NGPB)
              ELSE
                CALL CHI(STO,DSTO,LLGPB,NGPB)
                CALL RHO_SCREENED(PA,STO,DSTO,RHOA,LLGPB,NGPB)
                IF (OKS) CALL RHO_SCREENED(PB,STO,DSTO,RHOB,LLGPB,NGPB)
              END IF
C
C     *** Calculate atomic weights for grid points ***
C
              IF (FPA) THEN
                CALL FUZZY(WATOM,LLGPB,NGPB,NSPIN)
              ELSE IF (VPA) THEN
                CALL VORONOI(WATOM,LLGPB,NGPB,NSPIN)
              ELSE IF (HPA) THEN
                CALL HIRSHFELD(HTO,WATOM,LLGPB,NGPB,NSPIN)
              END IF
C
C     *** Add weighted rho values to atomic moments ***
C
              CALL ATOMRHO(WATOM,LLGPB,NGPB,NSPIN)
C
            END DO
C
          END DO
C
          DONE = .TRUE.
C
        END IF
C
      END DO
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(RADGRD,RADPTR,STAT=ALLOCATION)
C
      IF (TOPCD) THEN
        DEALLOCATE(HTO,WATOM,STAT=ALLOCATION)
      ELSE IF (HPA) THEN
        DEALLOCATE(HTO,STO,WATOM,STAT=ALLOCATION)
      ELSE
        DEALLOCATE(STO,WATOM,STAT=ALLOCATION)
      END IF
C
      CALL CHKERRSTAT(ALLOCATION,'NUMPOPDIR','DEALLOCATION FAILED')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NUMPOPDIR ***
C
      END
