      SUBROUTINE MINIMIZER(ESTEP,DE1STEP,OPTSTEP,DDEG,DUPD)
C
C     Purpose: Structure optimization by energy minimization.
C
C     History: - Creation (07.06.02, UR)
C                         (04.11.05, JMC)
C                         (20.04.17, GG)
C
C     ******************************************************************
C
C     List of local dimension:
C
C     DDEG: Dimension of degrees of freedom.
C     DUPD: Dimension of steps in Hessian update.
C
C     List of local variables:
C
C     ESTEP  : Energy of optimization steps.
C     DE1STEP: Internal energy gradients of optimization steps.
C     OPTSTEP: Internal coordinates of optimization steps.
C
C     List of local dynamical fields:
C
C     DE2   : Internal Hessian matrix -> Hessian eigenvectors. 
C     DRSTEP: Internal displacement step.
C     U     : Transformation matrix for active variables.
C
C     List of global variables:
C
C     DE: Cartesian energy gradients of optimization steps.
C     DR: Cartesian coordinates of optimization steps.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'iteration.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'opt.h'
C
      INTEGER DDEG,DUPD
      REAL DE1STEP(DDEG,DUPD),OPTSTEP(DDEG,DUPD)
C
      CHARACTER*80 PRTSTR,STRCOMP
C
      CHARACTER*80 STRING(5)
      REAL ESTEP(MAXUPD)
C
      INTEGER ALLOCATION,NPRI_ME,NVAR_ME,NDEG_ME
      REAL,ALLOCATABLE :: DE2(:,:),DRSTEP(:),U(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Synchronize parallel processes ***
C
      CALL MP_SYNC
C
C     *** Initialization ***
C
      OPTIMIZED = .FALSE.
      MINIMIZED = .FALSE.
      OPTRST = .FALSE.
      REJECT = .FALSE.
      REDOSTEP = .FALSE.
C
C     *** Set local dimensions ***
C
      NDEG_ME = NDEG
      NPRI_ME = NPRI
      NVAR_ME = NVAR
C
      IF (MPSLAVE) THEN
        NDEG_ME = 1
        NPRI_ME = 1
        NVAR_ME = 1
      END IF
C
C     *** Entering optimization loop ***
C
  500 CONTINUE
C
C     *** Optimization cycle output ***
C
      CALL OPTOUT
      IF (OPTIMIZED) RETURN
C
C     *** Allocate local fields ***
C
      ALLOCATE(U(NPRI_ME,NVAR_ME),DE2(NDEG_ME,NDEG_ME),DRSTEP(NDEG_ME),
     $         STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'MINIMIZER','ALLOCATION FAILED')
C
C     *** Load active space projector and Cartesian Hessian ***
C
      IF (MPMASTER) CALL OPTIO(U,NPRI*NVAR,'READ','PROJECTOR')
C
      IF (MPMASTER.AND.((OPTTYP.EQ.'CARTESIAN').OR.(OPTCYC.EQ.1).OR.
     $   (HESSIAN.EQ.'ANALYTIC').OR.(HESSIAN.EQ.'NUMERIC'))) THEN
        DE2(:,:) = 0.0 
        CALL OPTIO(DE2,NDEG**2,'READ','HESSIAN')
      END IF
C
C     *** Project external degrees of freedom ***
C     *** from Cartesian gradient and Hessian ***
C
      IF (OPTTYP.EQ.'CARTESIAN') THEN
C
        CALL PROJDE1(DE1STEP(1:NDEG_ME,1),U,NACT,NDEG,NPRI,NVAR)
        CALL PROJDE2(DE2,U,NDEG,NPRI,NVAR,'OPTIMIZE')
C
C     *** Transform Cartesian gradient and Hessian into active space ***
C
      ELSE
C
        CALL DE1TRANS(DE1STEP(:,1),U,NACT,NDEG,NPRI,NVAR)
        IF ((HESSIAN.EQ.'ANALYTIC').OR.(HESSIAN.EQ.'NUMERIC').OR.
     $      (OPTCYC.EQ.1)) THEN
          CALL DE2TRANS(DE1STEP(:,1),DE2,U,NACT,NDEG,NPRI,NVAR)
          IF (MPMASTER) CALL OPTIO(DE2,NDEG**2,'WRITE','DE2STEP')
        ELSE
          DE2(:,:) = 0.0
          IF (MPMASTER) CALL OPTIO(DE2,NDEG**2,'READ','DE2STEP')
          IF (MPMASTER) CALL UPDDE2(DE1STEP,DE2,OPTSTEP,NDEG,DUPD)
        END IF
C
      END IF
C
C     *** Set HESSIAN keyword for quasi-Newton method ***
C
      IF (.NOT.((UPHESS.EQ.'ANALYTIC').OR.(UPHESS.EQ.'NUMERIC'))) THEN
        HESSIAN = 'UPDATED'
      END IF  
C
C     *** Check current structure gradient ***
C
      IF (MPMASTER) THEN
        CALL CHKMIN(ESTEP,DE1STEP(:,1),OPTSTEP,STRING,NDEG,DUPD)
        CALL CONVOUT(STRING)
      END IF
C
      CALL MP_SEND_ALL(OPTIMIZED,1,MASTER,MP_LOG)
C
      IF (OPTIMIZED) THEN
        WRITE (PRTSTR,'("THE GEOMETRY IS OPTIMIZED")')
        CALL MESSAGE(1,STRCOMP(PRTSTR),1,1)
        DEALLOCATE(U,DE2,DRSTEP,STAT=ALLOCATION)
        CALL CHKERRSTAT(ALLOCATION,'MINIMIZER','DEALLOCATION FAILED')
        IF ((HESSIAN.EQ.'ANALYTIC').OR.(HESSIAN.EQ.'NUMERIC')) THEN
          HESSIAN = 'UPDATED'
        END IF
        CALL OPTSCF(ESTEP,'FINAL')
        RETURN
      END IF
C
C     *** Reload previous optimization step, if necessary ***
C
      IF (MPMASTER.AND.REDOSTEP) THEN
        REDOSTEP = .FALSE.
        CALL PRESTP(ESTEP,DE1STEP,OPTSTEP,NDEG,DUPD)
      END IF
C
C     *** Pick step type and perform optimization step ***
C
      CALL PICKSTP(DE1STEP(:,1),DE2,DRSTEP,NDEG,NDEG_ME)
C
C     *** Back transformation of displacement step ***
C
      CALL DRTRANS(DRSTEP,U,NACT,NDEG,NPRI_ME,NVAR_ME)
C
C     *** Store current optimization step ***
C
      CALL CURSTP(ESTEP,DE1STEP,OPTSTEP,U,DE2,NDEG,NPRI,NVAR,DUPD)
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(U,DE2,DRSTEP,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'MINIMIZER','DEALLOCATION FAILED')
C
C     *** Restart minimization if a bend becomes linear ***
C
      IF (MPMASTER) CALL CHKBEND
C
      CALL MP_SEND_ALL(OPTRST,1,MASTER,MP_LOG)
      CALL MP_SEND_ALL(REJECT,1,MASTER,MP_LOG)
C
      IF (OPTIMIZED.OR.OPTRST.OR.REJECT) RETURN
C
C     *** SCF for geometry optimization ***
C
      CALL OPTSCF(ESTEP,'STEP')
C
C     *** Calculate gradients of new geometry ***
C
      CALL GRADDRV(DE(:,1),NDEG)
C
CCC ADLL LA
C     *** Project out link atom forces ***
C
      CALL PRJLA(DE(:,1),NDEG,'OPT')
C
CCC ADLL LA
C
C     *** Update trust radius ***
C
      IF (MPMASTER) CALL UPDTRUST(ESTEP)
C
C     *** Update Cartesian Hessian matrix ***
C
      CALL UPDHES(NDEG)
C
C     *** Write new geometry step ***
C
      CALL NEWDRV('GEOMETRY STEP')
C
C     *** Print timer information ***
C
      CALL TIMER('STOP','OPTIMIZATION CYCLE')
      CALL TIMER('PRINT STEP','OPTIMIZATION CYCLE')
      CALL TIMER('START','OPTIMIZATION CYCLE')
C
      GO TO 500
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE MINIMIZER ***
C
      END
