      SUBROUTINE NUMCAPDIR(KSA,KSB,DSTO)
C
C     Purpose: Numerical calculation of the complex absorbing  
C              potential. The grid is retrieved (directly) on the fly.
C
C     History: - Creation (24.03.17, ADLL)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DRAD: Dimension of radial shells on the grid.
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     DA     : Diagonal of alpha matrix.
C     DB     : Diagonal of beta matrix.
C     LLATOM : Lower limit of atoms in current buffer (record).
C     LLBATCH: Lower limit of a grid point batch.
C     LLGPB  : Lower limit grid point of a batch.
C     NRAD   : Number of radial shells for the full grid.
C     ULATOM : Upper limit of atoms in current buffer (record).
C     ULBATCH: Upper limit of a grid point batch.
C     ULGPB  : Upper limit grid point of a batch.
C     VXCDCA : Alpha exchange-correlation potential vector.
C     VXCDCB : Beta exchange-correlation potential vector.
C     XCDCA  : Alpha charge density coefficients.
C     XCDCB  : Beta charge density coefficients.
C     
C     List of local dynamical fields:
C
C     D1HTO : First derivatives of Hermite typ orbitals.
C     D1STO : First derivatives of the basis functions.
C     HTO   : Function values of Hermite typ orbitals.
C     RADGRD: Radial PoinTs PoinTeR RADGRD(IRAD,N).
C             N = 1: Abscissa of IRAD.
C             N = 2: Radial weight of IRAD.
C     RADPTR: RADial PoinTeR RADPTR(IRAD,N).
C             N = 1: Atom of the current radial shell IRAD.
C             N = 2: Radial shell in the field NRADIAL.
C     STO   : Function values of the basis functions.
C     VGSTO : Work field for GGA exchange-correlation potential.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'rtdft.h'
C
      INTEGER DSTO
      REAL KSA(DSTO,DSTO),KSB(DSTO,DSTO)
C
      CHARACTER*80 TIMSTR
      LOGICAL DONE,STATED
      INTEGER IATOM,IBATCH,IRAD,JATOM,NGPB,NRAD
      INTEGER LLATOM,LLBATCH,LLGPB,LL_ME,ULATOM,ULBATCH,ULGPB,UL_ME
      REAL TLOCAL
C
      INTEGER ALLOCATION,DRAD
      INTEGER,ALLOCATABLE :: RADPTR(:,:)
      REAL,ALLOCATABLE :: RADGRD(:,:),STO(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Start timer for xc potential ***
C
      IF (MPP) THEN
        TIMSTR = 'MPI ABSORBING POTENTIAL'
      ELSE
        TIMSTR = 'ABSORBING POTENTIAL'
      END IF
C
      IF (.NOT.STATED('ATOM')) CALL TIMER('START',TIMSTR)
C
C     *** Allocate local fields for numerical integration ***
C
      DRAD = MAXRAD*NATOM
      ALLOCATE(RADGRD(DRAD,2),RADPTR(DRAD,2),STO(MAXGPB,DSTO),
     $         STAT=ALLOCATION)
C
      CALL CHKERRSTAT(ALLOCATION,'NUMABSDIR','ALLOCATION FAILED')
C
C     *** Initialize radial pointer, grid rotation and loop loads ***
C
      CALL RADINI(IATOM,NRAD,RADGRD,RADPTR,DRAD,LL_ME,UL_ME,MP_XCVE)
C
      IF (ROTGRD.AND.(LL_ME.GE.1)) THEN
        CALL GRIDROT(IATOM,RADGRD,RADPTR,DRAD,LL_ME,'STORE')
        CALL GRIDROT(IATOM,RADGRD,RADPTR,DRAD,LL_ME,'INITME')
      END IF
C
C     *** Loop over all radial shells ***
C
      DONE = .FALSE.
      CALL MPCPUTIM(TLOCAL,'START')
C
      DO IRAD=LL_ME,UL_ME
C
C     *** If necessary, update atom and radial pointer ***
C
        CALL RADUPD(IATOM,IRAD,RADGRD,RADPTR,DRAD,LL_ME,DONE,.TRUE.)
C
C     *** Radial sphere generation ***
C
        CALL RADGEN(IATOM,IRAD,RADGRD,RADPTR,DRAD,LL_ME,UL_ME,.TRUE.)
C
C     *** Do the numerical integration if buffer is filled ***
C
        IF (((NGPBUF+NGPSPH).GT.MAXGP).OR.(IRAD.EQ.UL_ME)) THEN
C
C     *** Set upper grid point limit for current atom ***
C
          ULGP(IATOM) = NGPBUF
C  
C     *** Save number of grid points for the last atom ***
C
          IF (ROTGRD.AND.(IRAD.EQ.UL_ME)) THEN
            CALL GRIDROT(IATOM,RADGRD,RADPTR,DRAD,LL_ME,'RESTORE')
            NGPVEC(IATOM) = NGPATOM
          ELSE IF (IRAD.EQ.UL_ME) THEN
            NGPVEC(IATOM) = NGPATOM
          END IF
C
C     *** Define lower and upper limits of atoms in buffer ***
C
          LLATOM = IATOM - NATOMBUF + 1
          ULATOM = LLATOM + NATOMBUF - 1
C
C     *** Loop over all atoms in current grid vector buffer ***
C
          DO JATOM=LLATOM,ULATOM
C
C     *** Find neighbors for JATOM ***
C
            CALL NEIGHBOR(JATOM)
C
C     *** Define lower and upper limits of ***
C     *** the current batch of grid points ***
C
            LLBATCH = (LLGP(JATOM) - 1)/MAXGPB + 1
            ULBATCH = (ULGP(JATOM) - 1)/MAXGPB + 1
C
C     *** Loop over all grid point batches of atom JATOM ***
C
            DO IBATCH=LLBATCH,ULBATCH
C
              IF (IBATCH.EQ.LLBATCH) THEN
                LLGPB = LLGP(JATOM)
                ULGPB = MIN(ULGP(JATOM),IBATCH*MAXGPB)
              ELSE IF (IBATCH.EQ.ULBATCH) THEN
                LLGPB = (IBATCH - 1)*MAXGPB + 1
                ULGPB = ULGP(JATOM)
              ELSE
                ULGPB = IBATCH*MAXGPB
                LLGPB = ULGPB - MAXGPB + 1
              END IF
C
              NGPB = ULGPB - LLGPB + 1
C
C     *** Calculate absorbing potential ***
C
              CALL CHI(STO,DSTO,LLGPB,NGPB)
              CALL NUMCAPV(JATOM,LLGPB,NGPB)
C
C     *** Add absorbing potential contribution of  ***
C     *** the current batch of grid points to the  ***
C     *** absorbing potential matrix.              ***
C
              CALL CAPV(KSA,STO,DSTO,VABS,NGPB)
C
            END DO
C
          END DO
C
          DONE = .TRUE.
C
        END IF
C
      END DO
C
C     *** Update parallel loop loads ***
C
      CALL MYXCLOAD(IATOM,NRAD,RADPTR,DRAD,TLOCAL,LL_ME,UL_ME,MP_XCVE,
     $              'UPDATE')
C
C     *** Collect results from processors ***
C
      IF (MPP) THEN
        CALL SYMMAT(KSA,DSTO,NSTO,'LOWUP')
        CALL MP_SUM(KSA,KSA,DSTO*DSTO,MP_REAL)
        IF (OKS) THEN
          CALL SYMMAT(KSB,DSTO,NSTO,'LOWUP')
          CALL MP_SUM(KSB,KSB,DSTO*DSTO,MP_REAL)
        END IF
      END IF
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(RADGRD,RADPTR,STO,STAT=ALLOCATION)
C
      CALL CHKERRSTAT(ALLOCATION,'NUMABSDIR','DEALLOCATION FAILED')
C
C     *** Stop timer for xc potential ***
C
      IF (.NOT.STATED('ATOM')) CALL TIMER('STOP',TIMSTR)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NUMCAPDIR ***
C
      END
