      SUBROUTINE RTDRV
C
C     Purpose: Driver routine for RTDFT calculation.
C
C     History: - Creation (09.09.15, XJ)
C                         (02.03.20, ADLL)
C
C     ******************************************************************
C
C     List of local variables:
C     CI    : imaginary MO coefficient matrix.
C     CR    : real MO coefficient matrix.
C     KI    : imaginary parts of Kohn-Sham matrix.
C     KR    : real parts of Kohn-Sham matrix.
C     PI    : imaginary parts of density matrix.
C     PR    : real parts of density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'eri.h'
      INCLUDE 'flags.h'
      INCLUDE 'interface.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'pointer.h'
      INCLUDE 'rtdft.h'
      INCLUDE 'timer.h'
C
      INCLUDE 'dynamics.h'
      INCLUDE 'embed.h'
      INCLUDE 'mmpot.h'
C    
      INCLUDE 'fileio.h' 
      INTEGER ALLOCATION,NSTO_ME
      REAL,ALLOCATABLE :: KAI(:,:),KBI(:,:),KAR(:,:),KBR(:,:)
      REAL,ALLOCATABLE :: PAI(:,:),PBI(:,:),PAR(:,:),PBR(:,:)
      REAL RDUMMY(1)
C
      LOGICAL MDFLUSH,SUCCESS
      INTEGER NOPTION
      CHARACTER*80 OPTIONS(MAXOPT)
C
C     ------------------------------------------------------------------
C
C     *** Quick return, if no excited states are requested ***
C
      IF (.NOT.RTDFT) RETURN
C
C     *** Build core Hamiltonian ***
C
      CALL BLDCORE
C
C     *** Allocate electron repulsion integral work vectors ***
C
      CALL SCFCHK
C
C     *** Allocate three-center ERI vector ***
C
      CALL PTRHANDLE(MAXERIS,ALLOCATION,'ALLOC',RPTRVEC=ERIVEC)
      CALL CHKERRSTAT(ALLOCATION,'SCFDRV','ERI ALLOCATION FAILED')
C
C     *** Allocate two-center near-field ERI vector ***
C
      CALL PTRHANDLE(MAXGERI,ALLOCATION,'ALLOC',RPTRVEC=GERIVEC)
      CALL CHKERRSTAT(ALLOCATION,'SCFDRV','GERI ALLOCATION FAILED')
C
C     *** Calculation of Coulomb integrals ***
C
      IF (ERITYP.NE.'DIRECT') THEN
        CALL BLDERIS
      ELSE IF (ERIRAM) THEN
        CALL MIXERIS
        CALL MIXGMAT
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Start timer for RT-TDDFT ***
C
      CALL TIMER('START','ELECTRON DYNAMICS SIMULATION ')
      CALL MESSAGE(2,'ELECTRON DYNAMICS SIMULATION SIMULATION',3,0)
C
C     *** Open external file for externam data sets ***
C
      IF (MLRT) OPEN (MLDAT,FILE='deMon_rt.dat',STATUS='UNKNOWN',
     $      ACCESS='SEQUENTIAL',FORM='FORMATTED')

C     *** Initialization *** 
C
      NSTO_ME = NSTO
      IF (MPSLAVE) NSTO_ME = 1
C
      IF (OKS) THEN
C
C     *** Open shell RT-TDDFT ***
C
        ALLOCATE(PAR(NSTO_ME,NSTO_ME),PBR(NSTO_ME,NSTO_ME),
     $           PBI(NSTO_ME,NSTO_ME),PAI(NSTO_ME,NSTO_ME),
     $           KAR(NSTO,NSTO),KAI(NSTO,NSTO),KBR(NSTO,NSTO),
     $           KBI(NSTO,NSTO),STAT=ALLOCATION)
C
        KAR(:,:) = 0.0
        KAI(:,:) = 0.0
        PAI(:,:) = 0.0
        PAR(:,:) = 0.0
        KBR(:,:) = 0.0
        KBI(:,:) = 0.0
        PBI(:,:) = 0.0
        PBR(:,:) = 0.0
C
        CALL OPRTINI(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,NSTO,NSTO_ME)
        CALL OPRT(KAR,KAI,KBR,KBI,PAR,PAI,PBR,PBI,NSTO,NSTO_ME)
C
C     *** Save WFN files ***
C
        CALL DOTDWFN(KAR,KAI,KBR,KBI,NSTO)
C
C     *** Plot time-dependent molecular fields ***
C
        CALL DOTDPLOT(KAR,KAI,KBR,KBI,NSTO)
C
        DEALLOCATE(PAR,PBR,PAI,PBI,KAR,KBR,KAI,KBI,STAT=ALLOCATION)
C
      ELSE
C
C     *** Close shell RT-TDDFT ***
C
        ALLOCATE(PAR(NSTO_ME,NSTO_ME),PAI(NSTO_ME,NSTO_ME),
     $          KAR(NSTO,NSTO),KAI(NSTO,NSTO),
     $          STAT=ALLOCATION)
C
        CALL CHKERRSTAT(ALLOCATION,'RTDRV','ALLOCATION FAILED') 
        KAR(:,:) = 0.0
        KAI(:,:) = 0.0
        PAI(:,:) = 0.0
        PAR(:,:) = 0.0
C
C     *** Perform real-time propagation *** 
C
        CALL CLRTINI(KAR,KAI,PAR,PAI,NSTO,NSTO_ME)
        CALL CLRT(KAR,KAI,PAR,PAI,NSTO,NSTO_ME)
C
C     *** Save WFN files ***
C
        CALL DOTDWFN(KAR,KAI,KAR,KAI,NSTO)
C
C     *** Plot time-dependent molecular fields ***
C
        CALL DOTDPLOT(KAR,KAI,KAR,KAI,NSTO)
C
        DEALLOCATE(PAR,PAI,KAR,KAI,STAT=ALLOCATION)
C
      END IF 
C
C     *** Delete electron repulsion integral scratch tape ***
C
      IF (MPMASTER.AND.(ERITYP.EQ.'ERI-I/O')) THEN
        CALL ERIIO(ERIVEC,MAXERIS,MAXERIS,'DELETE')
      END IF
C
C     *** Deallocate three-center ERI vector ***
C
      CALL PTRHANDLE(MAXERIS,ALLOCATION,'DEALLOC',RPTRVEC=ERIVEC)
      CALL CHKERRSTAT(ALLOCATION,'SCFDRV','ERI DEALLOCATION FAILED')
C
C     *** Deallocate two-center near-field ERI vector ***
C
      CALL PTRHANDLE(MAXGERI,ALLOCATION,'DEALLOC',RPTRVEC=GERIVEC)
      CALL CHKERRSTAT(ALLOCATION,'SCFDRV','GERI DEALLOCATION FAILED')
C
C     *** Turn off property flags ***
C
      ESA = .FALSE.
C
C     *** Close external file for externam data sets ***
C
      IF (MLRT) CLOSE (MLDAT)

C     *** Stop timer for RT-TDDFT ***
C
      CALL TIMER('STOP','ELECTRON DYNAMICS SIMULATION')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE RTDRV ***
C
      END
