      SUBROUTINE NUMXCE(LLGPB,NGPB)
C
C     Purpose: Numerical calculation of the exchange-correlation
C              energy.
C
C     History: - Creation (12.08.96, MK)
C                         (02.09.03, AVA)
C                         (05.03.20, DMR)
C                         (12.05.20, FAD)
C
C     ******************************************************************
C
C     List of local variables:
C
C     DRHO{ALP/BET}: Norm of the gradient of the alpha/beta density.
C     DRHOAB       : Product of alpha and beta density gradient.
C     DRHOTOT      : Norm of the gradient of the total density.
C     KIN{ALP/BET} : Kinetic energy density for alpha/beta spin.
C     NGPB         : Number of grid points in the current batch.
C     RHO{ALP/BET} : Alpha/Beta density.
C     RHOTOT       : Total electron density.
C     ZSPIN        : Relative spin polarization.
C
C     List of exchange energy functionals and enhancement factors:
C
C     EXLSD    : Uniform electron gas exchange functional.
C     FXALPHA  : X-alpha (Dirac) exchange enhancement factor.
C     FXPW86   : Perdew-Wang (1986) enhancement factor.
C     FXB88    : Becke (1988) enhancement factor.
C     FXPW91   : Perdew-Wang (1991) enhancement factor.
C     FXECMV   : Engel, Chevary, Macdonald & Vosko enhancement factor.
C     FXEV93   : Engel & Vosko (1993) enhancement factor.
C     FXPBE96  : Perdew, Burke & Ernzerhof (1996) enhancement factor.
C     FXPBE98  : Zhang & Yang PBE (1998) enhancement factor.
C     FXPBE99  : Hammer, Hansen & Norskov PBE (1999) enhancement factor.
C     FXPBESOL : PBEsol (2008) enhancement factor.
C     FXHC     : Handy & Cohen (2001) enhancement factor.
C     FXKT1    : Keal & Tozer (2003) enhancement factor 1.
C     FXKT2    : Keal & Tozer (2003) enhancement factor 2.
C     FXKT3    : Keal & Tozer (2004) enhancement factor 3.
C     FXVMT    : Vela, Medel & Trickey (2009) enhancement factor.
C     FXVMTSOL : VMTsol (2009) enhancement factor.
C     FXSO11   : Peverati, Zhao & Truhlar (2011) enhancement factor.
C     FXN12    : Peverati & Trular (2012) enhancement factor.
C     FXCAP    : Carmona-Espindola et al. CAP (2014) enhancement factor.
C     FXGAM    : Yu et al. (2015) enhancement factor.
C     FXVS98   : Van Voohris & Scuseria (1998) enhancement factor.
C     FXPKZB   : Perdew, Kurth, Zupan & Blaha (1999) enhancement factor.
C     FXTPSS   : Tao et al. (2003) enhancement factor.
C     FXM06    : Zhao & Truhlar (2006) enhancement factor.
C     FXM11    : Peverati & Trular (2011) enhancement factor.
C     FXMN12   : Peverati & Truhlar (2012) enhancement factor.
C     FXSCAN   : Sun, Ruzsinszky & Perdew (2015) enhancement factor.
C     FXB3LYP  : Stephens et al. (1994) B3LYP enhancement factor.
C     FXCAMB88 : Yanai et.al. (2004) CAMB3LYP enhancement factor.
C     FXCAMPBE : CAMPBE96 enhancement factor.
C     FXRCAMB88: Revised CAMB3LYP enhancement factor.
C
C
C     List of correlation energy functionals:
C
C     ECPVWN  : Pade interpolated VWN correlation functional (1980).
C     ECRVWN  : RPA interpolated VWN correlation functional (1980).
C     ECPZ81  : Perdew & Zunger correlation functional (1981).
C     ECPW92  : Perdew & Wang local correlation functional (1992).
C     ECP86   : Perdew correlation functional (1986).
C     ECLYP   : Lee-Yang-Parr correlation functional (1989).
C     ECPW91  : Perdew & Wang GGA correlation functional (1991).
C     ECPBESOL: PBEsol correlation functional (2008).
C     ECPBE   : Perdew, Burke & Ernzerhof correlation functional (1996).
C     ECSO11  : Peverati et al. GGA correlation functional (2011).
C     ECN12   : Peverati & Truhlar GGA correlation functional (2012).
C     ECGAM   : Yu et al. GGA type correlation functional (2015).
C     ECVS98  : Van Voohris et al. mGGA correlation functional (1998).
C     ECPKZB  : Perdew et al. mGGA correlation functional (1999).
C     ECTPSS  : Tao et al. mGGA correlation functional (2003).
C     ECM06   : Zhao & Truhlar mGGA correlation functional (2006).
C     ECM11   : Peverati & Truhlar mGGA correlation functional (2011).
C     ECMN12  : Peverati & Truhlar mGGA correlation functional (2012).
C     ECSCAN  : Sun et al. mGGA correlation functional (2015).
C     ECB3LYP : Stephens et al. hybrid correlation functional (1994).
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'threshold.h'
      INCLUDE 'tolerance.h'
      INCLUDE 'xc.h'
C
      INTEGER LLGPB,NGPB
C
      INTEGER IGP,NRHO
C
      REAL RHOALP(MAXGPB),RHOBET(MAXGPB),RHOTOT(MAXGPB)
      REAL DRHOALP(MAXGPB),DRHOBET(MAXGPB),DRHOTOT(MAXGPB)
      REAL DRHOAB(MAXGPB),KINALP(MAXGPB),KINBET(MAXGPB),ZSPIN(MAXGPB)
C
      REAL EXLSD,EXLSDSR
C
      REAL FXALPHA
      REAL FXPBESOL,FXVMTSOL
      REAL FXPW86,FXB88,FXPW91,FXECMV,FXEV93,FXPBE96,FXPBE98,FXPBE99
      REAL FXHC,FXKT1,FXKT2,FXKT3,FXVMT,FXSO11,FXN12,FXCAP,FXGAM
      REAL FXVS98,FXPKZB,FXTPSS,FXM06,FXM11,FXMN12,FXSCAN
      REAL FXB3LYP
      REAL FXCAMB88,FXCAMPBE,FXRCAMB88,FXHSE
cej
      REAL FXNCAP
cej
C
C
      REAL ECPVWN,ECRVWN,ECPZ81,ECPW92
      REAL ECP86,ECLYP,ECPW91,ECPBESOL,ECPBE,ECSO11,ECN12,ECGAM
      REAL ECVS98,ECPKZB,ECTPSS,ECM06,ECM11,ECMN12,ECSCAN
      REAL ECB3LYP
C
C     ------------------------------------------------------------------
C
C     *** Load density of the current batch of grid points ***
C
      IF (OKS) THEN
C
        NRHO = 2
        RHOALP(1:NGPB) = MAX(0.0,RHOA(1:NGPB))
        RHOBET(1:NGPB) = MAX(0.0,RHOB(1:NGPB))
        RHOTOT(1:NGPB) = RHOALP(1:NGPB) + RHOBET(1:NGPB)
        ZSPIN(1:NGPB) = (RHOALP(1:NGPB) - RHOBET(1:NGPB))/
     $                   MAX(RHOTOT(1:NGPB),TINIEST)
C
        EX(1:NGPB) = 0.0
        EC(1:NGPB) = 0.0
C
        IF (GGA.OR.LAP) THEN
          DRHOALP(1:NGPB) = SQRT(D1RHOA(1:NGPB,1)**2 +
     $                           D1RHOA(1:NGPB,2)**2 +
     $                           D1RHOA(1:NGPB,3)**2)
          DRHOBET(1:NGPB) = SQRT(D1RHOB(1:NGPB,1)**2 +
     $                           D1RHOB(1:NGPB,2)**2 +
     $                           D1RHOB(1:NGPB,3)**2)
          DRHOTOT(1:NGPB) = SQRT((D1RHOA(1:NGPB,1) + 
     $                            D1RHOB(1:NGPB,1))**2 +
     $                           (D1RHOA(1:NGPB,2) +
     $                            D1RHOB(1:NGPB,2))**2 +
     $                           (D1RHOA(1:NGPB,3) + 
     $                            D1RHOB(1:NGPB,3))**2)
          DRHOAB(1:NGPB) = D1RHOA(1:NGPB,1)*D1RHOB(1:NGPB,1) +
     $                     D1RHOA(1:NGPB,2)*D1RHOB(1:NGPB,2) +
     $                     D1RHOA(1:NGPB,3)*D1RHOB(1:NGPB,3)
        END IF
C
        IF (LAP) THEN
          KINALP(1:NGPB) = KINA(1:NGPB)
          KINBET(1:NGPB) = KINB(1:NGPB)
        END IF
C
      ELSE
C
        NRHO = 1
        RHOALP(1:NGPB) = MAX(0.0,0.5*RHOA(1:NGPB))
        RHOBET(1:NGPB) = RHOALP(1:NGPB)
        RHOTOT(1:NGPB) = RHOALP(1:NGPB) + RHOBET(1:NGPB)
        ZSPIN(1:NGPB) = 0.0
C
        EX(1:NGPB) = 0.0
        EC(1:NGPB) = 0.0
C
        IF (GGA.OR.LAP) THEN
          DRHOALP(1:NGPB) = 0.5*SQRT(D1RHOA(1:NGPB,1)**2 +
     $                               D1RHOA(1:NGPB,2)**2 +
     $                               D1RHOA(1:NGPB,3)**2)
          DRHOBET(1:NGPB) = DRHOALP(1:NGPB)
          DRHOTOT(1:NGPB) = DRHOALP(1:NGPB) + DRHOBET(1:NGPB)
          DRHOAB(1:NGPB) = DRHOALP(1:NGPB)**2
        END IF
C
        IF (LAP) THEN
          KINALP(1:NGPB) = 0.5*KINA(1:NGPB)
          KINBET(1:NGPB) = KINALP(1:NGPB)
        END IF
C
      END IF
C
C     ------------------------------------------------------------------
C
      CALL LXC_XCE(RHOALP,RHOBET,RHOTOT,DRHOALP,DRHOBET,DRHOTOT,DRHOAB,
     $             KINALP,KINBET,EX,EC,GRDVEC(4,LLGPB:LLGPB+NGPB-1),
     $             EXSUM,ECSUM,NGPB,NRHO)
C
C     ------------------------------------------------------------------
C
C     *** Calculate exchange energy contribution ***
C
      IF (XTYP(1:6).EQ.'XALPHA') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*FXALPHA(RHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*FXALPHA(RHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'LSD') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PW86') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXPW86(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXPW86(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'B88') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXB88(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXB88(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PW91') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXPW91(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXPW91(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'ECMV') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXECMV(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXECMV(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'EV93') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXEV93(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXEV93(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'PBE96') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXPBE96(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXPBE96(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'PBE98') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXPBE98(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXPBE98(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'PBE99') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXPBE99(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXPBE99(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'PBESOL') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXPBESOL(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXPBESOL(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'OPTX') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXHC(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXHC(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'KT1X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXKT1(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXKT1(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'KT2X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXKT2(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXKT2(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'KT3X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXKT3(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXKT3(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'VMT09') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXVMT(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXVMT(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'VMTSOL') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXVMTSOL(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXVMTSOL(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'SO11') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXSO11(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXSO11(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'N12') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*FXN12(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXN12(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'CAP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXCAP(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXCAP(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
cej
      ELSE IF (XTYP(1:5).EQ.'XNCAP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXNCAP(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXNCAP(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'GAM') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*FXGAM(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXGAM(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'VS98') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXVS98(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),1)
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXVS98(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),1)
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PKZB') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXPKZB(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXPKZB(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO 
C
      ELSE IF (XTYP(1:4).EQ.'TPSS') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXTPSS(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXTPSS(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'M06L') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXM06(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),1)
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXM06(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),1)
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'M11') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSDSR(RHOALP(IGP),0.25)*
     $                FXM11(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),1)
            EX(IGP) = EX(IGP) + EXLSDSR(RHOBET(IGP),0.25)*
     $                FXM11(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),1)
            EX(IGP) = EX(IGP) +
     $                (EXLSD(RHOALP(IGP)) - EXLSDSR(RHOALP(IGP),0.25))*
     $                FXM11(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),2)
            EX(IGP) = EX(IGP) +
     $                (EXLSD(RHOBET(IGP)) - EXLSDSR(RHOBET(IGP),0.25))*
     $                FXM11(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),2)
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'MN12') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXMN12(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXMN12(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'SCAN') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXSCAN(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXSCAN(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'B3LYP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXB3LYP(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXB3LYP(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PBE0') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXPBE96(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXPBE96(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*(1.0-PEXX)*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'M062X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXM06(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),4)
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXM06(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),4)
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'M06HF') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXM06(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),2)
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXM06(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),2)
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'M06') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXM06(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),3)
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXM06(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),3)
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'CAMB88') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXCAMB88(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXCAMB88(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:7).EQ.'RCAMB88') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXRCAMB88(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXRCAMB88(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'CAMPBE') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXCAMPBE(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXCAMPBE(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'HSE') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EX(IGP) = EXLSD(RHOALP(IGP))*
     $                FXHSE(RHOALP(IGP),DRHOALP(IGP))
            EX(IGP) = EX(IGP) + EXLSD(RHOBET(IGP))*
     $                FXHSE(RHOBET(IGP),DRHOBET(IGP))
            EX(IGP) = 0.5*EX(IGP)
          END IF
          EXSUM = EXSUM + GRDVEC(4,LLGPB+IGP-1)*EX(IGP)
        END DO

C
      ELSE IF (XTYP(1:5).NE.'LIBXC') THEN
C
        EXSUM = 0.0
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Calculate correlation energy contribution ***
C
      IF (CTYP(1:4).EQ.'PVWN') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECPVWN(RHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'RVWN') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECRVWN(RHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'PZ81') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECPZ81(RHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'PW92') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECPW92(RHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'P86') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECP86(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'LYP') THEN
C
        DO IGP=1,NGPB
          IF ((RHOALP(IGP).GT.GPTHRESH(IGP)).AND.
     $        (RHOBET(IGP).GT.GPTHRESH(IGP))) THEN
            EC(IGP) = ECLYP(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                      DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'PW91') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECPW91(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:6).EQ.'PBESOL') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECPBESOL(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF ((CTYP(1:3).EQ.'PBE').OR.(CTYP(1:4).EQ.'SPBE')) THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECPBE(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'KT2C') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = BETAKT2*ECPVWN(RHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'KT3C') THEN
C
        DO IGP=1,NGPB
          IF ((RHOALP(IGP).GT.GPTHRESH(IGP)).AND.
     $        (RHOBET(IGP).GT.GPTHRESH(IGP))) THEN
            EC(IGP) = BETAKT3*ECLYP(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                        DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'SO11') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECSO11(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'N12') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECN12(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                      DRHOALP(IGP),DRHOBET(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'GAM') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECGAM(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                      DRHOALP(IGP),DRHOBET(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'VS98') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECVS98(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                      DRHOALP(IGP),DRHOBET(IGP),KINALP(IGP),
     $                      KINBET(IGP),ZSPIN(IGP),1)
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'PKZB') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECPKZB(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                       DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP),
     $                       KINALP(IGP),KINBET(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'TPSS') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECTPSS(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                       DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP),
     $                       DRHOAB(IGP),KINALP(IGP),KINBET(IGP),
     $                       ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'M06L') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECM06(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                      DRHOALP(IGP),DRHOBET(IGP),KINALP(IGP),
     $                      KINBET(IGP),ZSPIN(IGP),1)
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'M11') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECM11(RHOTOT(IGP),DRHOTOT(IGP),KINALP(IGP),
     $                      KINBET(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'MN12') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECMN12(RHOTOT(IGP),DRHOTOT(IGP),KINALP(IGP),
     $                       KINBET(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'SCAN') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECSCAN(RHOTOT(IGP),DRHOTOT(IGP),KINALP(IGP),
     $                       KINBET(IGP),ZSPIN(IGP))
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:6).EQ.'RB3LYP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = (1.0 - PLYP)*ECRVWN(RHOTOT(IGP),ZSPIN(IGP))
            IF ((RHOALP(IGP).GT.GPTHRESH(IGP)).AND.
     $          (RHOBET(IGP).GT.GPTHRESH(IGP))) THEN
              EC(IGP) = EC(IGP) + ECB3LYP(RHOALP(IGP),RHOBET(IGP),
     $                                    RHOTOT(IGP),DRHOALP(IGP),
     $                                    DRHOBET(IGP),DRHOTOT(IGP))
            END IF
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:6).EQ.'PB3LYP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = (1.0 - PLYP)*ECPVWN(RHOTOT(IGP),ZSPIN(IGP))
            IF ((RHOALP(IGP).GT.GPTHRESH(IGP)).AND.
     $          (RHOBET(IGP).GT.GPTHRESH(IGP))) THEN
              EC(IGP) = EC(IGP) + ECB3LYP(RHOALP(IGP),RHOBET(IGP),
     $                                    RHOTOT(IGP),DRHOALP(IGP),
     $                                    DRHOBET(IGP),DRHOTOT(IGP))
            END IF
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:5).EQ.'M062X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECM06(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                      DRHOALP(IGP),DRHOBET(IGP),KINALP(IGP),
     $                      KINBET(IGP),ZSPIN(IGP),4)
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:5).EQ.'M06HF') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECM06(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                      DRHOALP(IGP),DRHOBET(IGP),KINALP(IGP),
     $                      KINBET(IGP),ZSPIN(IGP),2)
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'M06') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            EC(IGP) = ECM06(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                      DRHOALP(IGP),DRHOBET(IGP),KINALP(IGP),
     $                      KINBET(IGP),ZSPIN(IGP),3)
            ECSUM = ECSUM + GRDVEC(4,LLGPB+IGP-1)*EC(IGP)
          END IF
        END DO
C
      ELSE IF (CTYP(1:5).NE.'LIBXC') THEN
C
        ECSUM = 0.0
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NUMXCE ***
C
      END
