      SUBROUTINE CPSDRV
C
C     Purpose: Driver routine for critical point search.
C
C     History: - Creation (18.03.03, AMK)
C                         (09.02.17, GG)
C
C     ******************************************************************
C
C     List of local dynamical fields:
C
C     CA: Alpha MO coefficient matrix.
C     CB: Beta MO coefficient matrix.
C     PA: Alpha density matrix.
C     PB: Beta density matrix.
C
C     External functions:
C
C     WD1ESP: Calculate electrostatic potential gradient (Wrapper).
C     WD2ESP: Calculate potential 2nd derivative (Wrapper).
C
C     WD1RHO: Calculate molecular density gradient (Wrapper).
C     WD2RHO: Calculate molecular density 2nd derivative (Wrapper).
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'plot.h'
      INCLUDE 'scfvec.h'
C
      REAL CDC(MAXAUX)
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: CA(:,:),CB(:,:),PA(:,:),PB(:,:)
C
      EXTERNAL WD1ESP,WD2ESP,WD1RHO,WD2RHO
C
C     ------------------------------------------------------------------
C
C     *** Normalization of fitting coefficients ***
C
      IF (USEDD) CALL NORMCDC(CDCA,CDCB,NAUX)
C
C     *** Select MOs for molecular scalar field ***
C
CCC ADLL
      CALL GETOMOS(EIGVALA,HFOMOA,NPLOTFUN,PLOTMOA,PLOTMOSA)
      IF (OKS) CALL GETOMOS(EIGVALB,HFOMOB,NPLOTFUN,PLOTMOB,PLOTMOSB)
CCC ADLL
C
      CALL CHKPLOT('ORBITALS')
C
C     *** Set scalar molecular field name ***
C
      CALL PIEDRV('PIENAME')
C
C     *** Allocate local fields ***
C
      ALLOCATE(CA(NSTO,NSTO),PA(NSTO,NSTO),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'CPSDRV','ALPHA ALLOCATION FAILED')
      IF (OKS) THEN
        ALLOCATE(CB(NSTO,NSTO),PB(NSTO,NSTO),STAT=ALLOCATION)
        CALL CHKERRSTAT(ALLOCATION,'CPSDRV','BETA ALLOCATION FAILED')
      END IF
C
C     *** Build scalar molecular field density matrix ***
C
      IF (MPMASTER) THEN
        CALL RSTIO(CA,EIGVALA,PA,NSTO,NSTO,OCCNA,AOCCNA,ELCFGA,
     $             MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
      END IF
C
      CALL MP_SEND_ALL(CA,NSTO*NSTO,MASTER,MP_REAL)
      CALL BLDPLOT(CA,PA,NSTO,OCCNA,HFOMOA,PLOTMOSA)
C
      IF (OKS) THEN
        IF (MPMASTER.AND.ROKS) THEN
          CB(:,:) = CA(:,:)
        ELSE IF (MPMASTER) THEN
          CALL RSTIO(CB,EIGVALB,PB,NSTO,NSTO,OCCNB,AOCCNB,ELCFGB,
     $               MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'READ','BETA')
        END IF
        CALL MP_SEND_ALL(CB,NSTO*NSTO,MASTER,MP_REAL)
        CALL BLDPLOT(CB,PB,NSTO,OCCNB,HFOMOB,PLOTMOSB)
      END IF
C
C     *** Build deformation density, if requested ***
C
      IF (OKS.AND.USEDD) THEN
        CALL BLDDRHO(PA,PB,NSTO)
      ELSE IF (USEDD) THEN
        CALL BLDDRHO(PA,CA,NSTO)
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Search critical points of the electronic density ***
C
      IF (PLOTFNC.EQ.'RHO') THEN
C
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTCD) CDC(:) = CDCA(:) + CDCB(:)
        CALL MP_SEND_ALL(CDC,NAUX,MASTER,MP_REAL)
        IF (OKS) THEN
          CALL SEARCHER(CA,CB,PA,PB,CDC,NSTO,WD1RHO,WD2RHO)
        ELSE
          CALL SEARCHER(CA,CA,PA,PA,CDC,NSTO,WD1RHO,WD2RHO)
        END IF
C
C     *** Search critical points of the electrostatic potential ***
C
      ELSE IF (PLOTFNC.EQ.'ESP') THEN
C
        IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
        IF (PLOTCD) THEN
          CDC(:) = CDCA(:) + CDCB(:)
          CALL BLDCDC(CDC,'NORMALIZE')
          CALL MP_SEND_ALL(CDC,NAUX,MASTER,MP_REAL)
        ELSE
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'TOTAL',2.0)
          CALL SCALMAT(PA,NSTO,NSTO,NSTO,'M = N',0.5)
        END IF
        IF (OKS) THEN
          CALL SEARCHER(CA,CB,PA,PB,CDC,NSTO,WD1ESP,WD2ESP)
        ELSE
          CALL SEARCHER(CA,CA,PA,PA,CDC,NSTO,WD1ESP,WD2ESP)
        END IF
C
C     *** No more molecular fields ***
C
      ELSE
C
        CALL ERRMSG('CPSDRV','INVALID MOLECULAR FIELD',1)
C
      END IF
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(CA,PA,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'CPSDRV','ALPHA DEALLOCATION FAILED')
      IF (OKS) THEN
        DEALLOCATE(CB,PB,STAT=ALLOCATION)
        CALL CHKERRSTAT(ALLOCATION,'CPSDRV','BETA DEALLOCATION FAILED')
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE CPSDRV ***
C
      END
