      SUBROUTINE DUNLAP
C
C     Purpose: Calculation of charge density electron repulsion
C              integral (ERI) matrix.
C
C     Lit.: B.I. Dunlap et al., J. Chem. Phys. 71, 4993 (1979)
C
C     History: - Creation (18.03.99, AMK)
C                         (03.11.05, FJ)
C                         (17.10.12, GG)
C                         (25.04.18, NPM)
C                         (19.10.18, AMK)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DAUX   : Dimension of auxiliary function matrix.
C     DAUX_ME: Local dimension of auxiliary function matrix on node ME.
C
C     List of local dynamical fields:
C
C     CDERI: Charge density ERI matrix -> Inverse ERI matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'auxis.h'
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
C
      LOGICAL STATED
      CHARACTER*80 PRTSTR,TIMSTR
      INTEGER IAUX
C
      INTEGER IDUM,NAUX_ME,MAUX_ME
C
      INTEGER ALLOCATION,DAUX,DAUX_ME
      REAL,ALLOCATABLE :: CDERI(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Quick return, if all auxiliary functions are fixed ***
C
      IF (NAUX.EQ.0) RETURN
C
C     *** Synchronize parallel processes ***
C
      CALL MP_SYNC
C
C     *** Start timer for auxiliary function fit ***
C
      IF (MPP) THEN
        TIMSTR = 'MPI AUXILIARY FUNCTION FIT'
      ELSE
        TIMSTR = 'AUXILIARY FUNCTION FIT'
      END IF
C
      IF (.NOT.STATED('ATOM')) THEN
        CALL TIMER('START',TIMSTR)
      END IF
C
C     *** Determine addressing of CDERI in BLDIMAT ***
C
      USEMPF = (MPP.AND.(.NOT.STATED('ATOM')))
C
      IF (.NOT.USEMPF) THEN
        DAUX = NAUX
        DAUX_ME = NAUX
        NAUX_ME = NAUX
      ELSE
        DAUX = NAUX
        DAUX_ME = NAUX/NCPU + 1
        CALL MPGETINFO(NAUX,3,ME,NAUX_ME,LLROW,ULROW,IDUM)
      END IF
C
C     *** Allocate charge density ERI matrix or vector ***
C
      IF (USESVD) THEN
        ALLOCATE(CDERI(DAUX_ME,DAUX),STAT=ALLOCATION)
      ELSE
        ALLOCATE(CDERI(NADB,1),STAT=ALLOCATION)
      END IF
      CALL CHKERRSTAT(ALLOCATION,'DUNLAP','ALLOCATION FAILED')
C
C     *** Calculate fitted charge density ERI matrix ***
C
      IF (USESVD) THEN
        CALL BLDGMAT(CDERI,DAUX_ME,DAUX)
      ELSE
        CALL BLDGDIA(CDERI,NADB)
      END IF
C
C     *** Generate ERI matrix near-field atom pointer ***
C
      CALL GENGAT
C
C     *** Symmetrize ERI matrix ***
C
      IF (USEMPF.AND.USESVD) THEN
        CALL MPSYMMAT(CDERI,DAUX_ME,NAUX,'LOWUP')
      ELSE IF (USESVD) THEN
        CALL SYMMAT(CDERI,DAUX,NAUX,'UPLOW')
      END IF
C
C     *** Store charge density ERI matrix on tape ***
C
      IF (USESVD) THEN
        DO IAUX=1,NAUX_ME
          CALL CDFIO(CDERI(IAUX,1:NAUX),NAUX,IAUX,'WRITE','CDERI')
        END DO
      END IF
C
C     *** Print charge density ERI matrix ***
C
      IF ((.NOT.USEMPF).AND.USESVD.AND.PRTAMAT) THEN
        IF (STATUS.EQ.'SCF') THEN
          PRTSTR = 'FITTED DENSITY ERI MATRIX'
          CALL PRIMAT(CDERI,DAUX,1,NAUX,1,NAUX,2,5,OUT,'NONE','TOTAL',
     $                PRTSTR)
        END IF
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Calculate inverse ERI matrix, if necessary ***
C
      IF ((STATUS.EQ.'SCF').OR.(STATUS.EQ.'SCANSCF').OR.
     $    (STATUS.EQ.'POLAR').OR.(STATUS.EQ.'TDDFT').OR.
     $    (STATUS.EQ.'FFP').OR.(STATUS.EQ.'SADSCF').OR.
     $    (STATUS.EQ.'FERMI').OR.STATED('ATOM').OR.STATED('END').OR.
CCC ADLL
     $    (STATUS.EQ.'RTDFT').OR.
CCC ADLL
     $    ((STATUS.EQ.'IRC').AND.(HESSIAN.NE.'CALCULATED'))) THEN

C
C     *** Invert charge density ERI matrix ***
C
        IF (USESVD) CALL INVCDERI(CDERI,DAUX,DAUX_ME)
C
C     *** Print inverse charge density ERI matrix ***
C
        IF ((.NOT.USEMPF).AND.USESVD.AND.PRTAMAT) THEN
          IF (USEDIA.AND.(STATUS.EQ.'SCF')) THEN
            PRTSTR = 'INVERSE FITTED DENSITY ERI EIGENVALUES'
            CALL PRIVEC(EIGAUX,1,NAUX,8,2,OUT,PRTSTR)
          ELSE IF (STATUS.EQ.'SCF') THEN
            PRTSTR = 'INVERSE FITTED DENSITY ERI MATRIX'
            CALL PRIMAT(CDERI,DAUX,1,NAUX,1,NAUX,5,2,OUT,'NONE',
     $                  'TOTAL',PRTSTR)
          END IF
        END IF
C
C     *** Store inverse charge density ERI matrix on CDF tape ***
C
        IF (FITTING.EQ.'ANALYTICAL') THEN
C
          IF (.NOT.USEDIA) THEN
            DO IAUX=1,NAUX_ME
              CALL CDFIO(CDERI(IAUX,1:NAUX),NAUX,IAUX,'WRITE','ICDERI')
            END DO
          END IF
C
C     *** Store inverse charge density ERI matrix on PCG tape ***
C
        ELSE IF (FITTING.EQ.'NUMERICAL') THEN
C
          IF (USEMPF.AND.USESVD) THEN
            CALL MPGETINFO(NAUX,-1,ME,MAUX_ME,IDUM,IDUM,IDUM)
          ELSE IF (USESVD) THEN
            MAUX_ME = NAUX*(NAUX + 1)/2
          END IF
          IF (USESVD) CALL PCGSORT(CDERI,DAUX_ME,DAUX)
          IF (USESVD) CALL PCGIO(CDERI,MAUX_ME,1,'WRITE')
C
        END IF
C
C     *** Initialize charge density coefficient vectors ***
C
        FCDA(1:NALL) = 0.0
        FCDB(1:NALL) = 0.0
C
C     *** Initialize exchange-correlation start vectors ***
C
        XCD0A(1:NALL) = 0.0
        XCD0B(1:NALL) = 0.0
C
C     *** Calculate inverse ERI matrix for analytical fitting ***
C
      ELSE IF (FITTING.EQ.'ANALYTICAL') THEN
C
C     *** Invert charge density ERI matrix ***
C
        IF (USESVD) CALL INVCDERI(CDERI,DAUX,DAUX_ME)
C
C     *** Store inverse charge density ERI matrix on CDF tape ***
C
        IF (.NOT.USEDIA) THEN
          DO IAUX=1,NAUX_ME
            CALL CDFIO(CDERI(IAUX,1:NAUX),NAUX,IAUX,'WRITE','ICDERI')
          END DO
        END IF
C
      END IF
C
C     *** Calculate fixed charge density contributions ***
C
      IF (.NOT.STATED('ATOM')) CALL DUNLAP_FIX
C
C     *** Deallocate local fields ***
C
      IF (USESVD) THEN
        DEALLOCATE(CDERI,STAT=ALLOCATION)
        CALL CHKERRSTAT(ALLOCATION,'DUNLAP','DEALLOCATION FAILED')
      END IF
C
C     *** Stop timer for auxiliary function fit ***
C
      IF (.NOT.STATED('ATOM')) THEN
        CALL TIMER('STOP',TIMSTR)
        IF (STATUS.EQ.'SCF') CALL TIMER('PRINT STEP',TIMSTR)
      END IF
C
      USEMPF = .FALSE.
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE DUNLAP ***
C
      END
