      SUBROUTINE NUMNORM(NCDCA,NCDCB,NALPHA,NBETA,DAUX)
C
C     Purpose: Normalize numerical fitting coefficients.
C
C     History: - Creation (07.09.12, AMK)
C
C     ******************************************************************
C
C     List of local dimension:
C
C     DAUX: Dimension of auxiliary function matrix.
C     DHTD: Dimension of inverse Hessian in tridiagonal packed form.
C
C     List of local variables:
C
C     LAMBD{A/B}: Undefined Lagrange multipliers.
C     NCDC{A/B} : Normalized charge density fitting coefficients.
C
C     List of local dynamical fields:
C
C     H     : Approximated inverse of the charge density matrix.
C     RCDCA : Reduced alpha charge density coefficients.
C     RCDCB : Reduced beta charge density coefficients.
C     RQAUX : Reduced charge moments of auxiliary functions.
C     VECTOR: Work vector.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
C
      INTEGER DAUX
      REAL NCDCA(DAUX),NCDCB(DAUX)
C
      INTEGER IAUX,IDUM
      REAL A,B,LAMBDA,LAMBDB,NALPHA,NBETA,TERM
C
      INTEGER ALLOCATION,DHTD
      REAL,ALLOCATABLE :: H(:),RCDCA(:),RCDCB(:),RQAUX(:),VECTOR(:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      IF (USEMIN) RETURN
      IF (USEMPF) THEN
        CALL MPGETINFO(NAUX,-1,ME,DHTD,IDUM,IDUM,IDUM)
      ELSE
        DHTD = NAUX*(NAUX+1)/2
      END IF
C
      ALLOCATE(H(DHTD),RCDCA(NAUX),RCDCB(NAUX),RQAUX(NAUX),
     $         VECTOR(NAUX),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'NUMNORM','ALLOCATION FAILED')
C
C     *** Load triangular part of approximated inverse G matrix ***
C
      CALL PCGIO(H,DHTD,1,'READ')
C
C     *** Use diagonal representation of fit equation ***
C
      IF (USEDIA) THEN
C
C     *** Transform fitting coefficients to reduced SVD space ***
C
        CALL TRACDV(NCDCA,RCDCA,VECTOR,NAUX,'REDUCE')
        CALL TRACDV(NCDCB,RCDCB,VECTOR,NAUX,'REDUCE')
C
C     *** Transform charge moments to reduced SVD space ***
C
        CALL TRACDV(QAUX,RQAUX,VECTOR,NAUX,'REDUCE')
C
C     *** Calculate undefined Lagrange multiplier ***
C
        A = 0.0
        B = 0.0
        TERM = 0.0
C
        IF (USEMPF) THEN
          CALL MPMULVEC(H,RQAUX,VECTOR,DHTD,NAUX,NAUX,'SYMLO','NONE',
     $                  .TRUE.)
        ELSE
          CALL MULVEC(H,RQAUX,VECTOR,DHTD,NAUX,NAUX,'SYMLO','NONE',
     $                .TRUE.)
        END IF
C
        DO IAUX=1,NAUX
          A = A + RQAUX(IAUX)*RCDCA(IAUX)
          B = B + RQAUX(IAUX)*RCDCB(IAUX)
          TERM = TERM + RQAUX(IAUX)*VECTOR(IAUX)
        END DO
C
        LAMBDA = (A - NALPHA)/TERM
        LAMBDB = (B - NBETA)/TERM
C
C     *** Calculate normalized coefficients in reduced SVD space ***
C
        DO IAUX=1,NAUX
          RCDCA(IAUX) = RCDCA(IAUX) - LAMBDA*VECTOR(IAUX)
          RCDCB(IAUX) = RCDCB(IAUX) - LAMBDB*VECTOR(IAUX)
        END DO
C
C     *** Transform reduced normalized fitting coefficient back ***
C
        CALL TRACDV(NCDCA,RCDCA,VECTOR,NAUX,'EXPAND')
        CALL TRACDV(NCDCB,RCDCB,VECTOR,NAUX,'EXPAND')
C
C     *** Use non-diagonal representation of fit equation ***
C
      ELSE
C
C     *** Calculate undefined Lagrange multiplier ***
C
        A = 0.0
        B = 0.0
        TERM = 0.0
C
        IF (USEMPF) THEN
          CALL MPMULVEC(H,QAUX,VECTOR,DHTD,NAUX,NAUX,'SYMLO','NONE',
     $                  .TRUE.)
        ELSE
          CALL MULVEC(H,QAUX,VECTOR,DHTD,NAUX,NAUX,'SYMLO','NONE',
     $                .TRUE.)
        END IF
C
        DO IAUX=1,NAUX
          A = A + QAUX(IAUX)*NCDCA(IAUX)
          B = B + QAUX(IAUX)*NCDCB(IAUX)
          TERM = TERM + QAUX(IAUX)*VECTOR(IAUX)
        END DO
C
        LAMBDA = (A - NALPHA)/TERM
        LAMBDB = (B - NBETA)/TERM
C
C     *** Calculate normalized fitting coefficients ***
C
        DO IAUX=1,NAUX
          NCDCA(IAUX) = NCDCA(IAUX) - LAMBDA*VECTOR(IAUX)
          NCDCB(IAUX) = NCDCB(IAUX) - LAMBDB*VECTOR(IAUX)
        END DO
C
      END IF
C
C     *** Deallocate local fields ***
C
      DEALLOCATE(H,RCDCA,RCDCB,RQAUX,VECTOR,STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'NUMNORM','DEALLOCATION FAILED')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NUMNORM ***
C
      END
