      SUBROUTINE QMGRAD(GRADIENT,DDEG)
C
C     Purpose: Driver routine for QM energy gradient calculation.
C
C     History: - Creation (14.12.99, AMK)
C                         (01.05.05, AG)
C                         (27.11.13, DMR)
C                         (08.02.18, GG)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DDEG: Number of degrees of freedom, including dummy atoms.
C
C     List of local variables:
C
C     GRADIENT: Energy gradient of current step.
C     NCDCA   : Normalized alpha charge density coefficients.
C     NCDCB   : Normalized beta charge density coefficients.
C
C     List of local dynamical fields:
C
C     CA: Alpha MO coefficient matrix.
C     CB: Beta MO coefficient matrix.
C     PA: Alpha density matrix.
C     PB: Beta density matrix.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
CCC ADLL
      INCLUDE 'embed.h'
CCC ADLL
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
CCC ADLL
      INCLUDE 'rtdft.h'
CCC ADLL
      INCLUDE 'scfvec.h'
     
C
      INTEGER DDEG
      REAL GRADIENT(DDEG)
C
      CHARACTER*80 PRTSTR
C
      REAL NCDCA(MAXAUX),NCDCB(MAXAUX)
C
      INTEGER ALLOCATION,NSTO_ME
      REAL,ALLOCATABLE :: CA(:,:),CB(:,:),PA(:,:),PB(:,:)
C
      INTEGER IFIELD
CCC ADLL
      REAL EFMOD1,EFMOD2,FIELD
CCC ADLL
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields ***
C
      NSTO_ME = NSTO
      IF (MPSLAVE) NSTO_ME = 1
C
      IF (OKS) THEN
        ALLOCATE(CA(NSTO_ME,NSTO_ME),PA(NSTO,NSTO),CB(NSTO_ME,NSTO_ME),
     $           PB(NSTO,NSTO),STAT=ALLOCATION)
      ELSE
        ALLOCATE(CA(NSTO_ME,NSTO_ME),PA(NSTO,NSTO),STAT=ALLOCATION)
      END IF
      CALL CHKERRSTAT(ALLOCATION,'QMGRAD','ALLOCATION FAILED')
C
C     *** Calculate nuclei repulsion forces ***
C
      IF (MPMASTER) CALL D1NUCREP(GRADIENT,DDEG/3)
C
C     *** Calculate empirical van der Waals forces ***
C
      IF (MPMASTER) CALL D1VDW(GRADIENT,DDEG/3)
C
C     *** Block diagonalize ROKS MO coefficients ***
C
      IF (ROKS) CALL BLKROMO(PA,PB,CA,CB,NSTO,NSTO_ME)
C
      IF (MPSLAVE) GO TO 100
C
C     *** Read and restore MO coefficients ***
C
      IF (ROKS) THEN
        CA(:,:) = PA(:,:)
        CB(:,:) = PB(:,:)
      ELSE IF (OKS) THEN
        CALL RSTIO(CA,EIGVALA,PA,NSTO_ME,NSTO,OCCNA,AOCCNA,ELCFGA,
     $             MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
        CALL RSTIO(CB,EIGVALB,PB,NSTO_ME,NSTO,OCCNB,AOCCNB,ELCFGB,
     $             MOEXCB,NMOEXCB,LFOMOB,HFOMOB,'READ','BETA')
      ELSE
        CALL RSTIO(CA,EIGVALA,PA,NSTO_ME,NSTO,OCCNA,AOCCNA,ELCFGA,
     $             MOEXCA,NMOEXCA,LFOMOA,HFOMOA,'READ','ALPHA')
      END IF
C
C     *** Build energy weighted density matrix ***
C
      PA(:,:) = 0.0
      CALL BLDWMAT(CA,PA,NSTO,EIGVALA,OCCNA,HFOMOA)
      IF (OKS) CALL BLDWMAT(CB,PA,NSTO,EIGVALB,OCCNB,HFOMOB)
C
C     *** Print energy weighted density matrix ***
C
      IF (PRTPMAT) THEN
        PRTSTR = 'ENERGY WEIGHTED DENSITY MATRIX'
        CALL PRIMAT(PA,NSTO,1,NSTO,1,NSTO,2,5,OUT,'BASIS','TOTAL',
     $              PRTSTR)
      END IF
C
C     *** Calculate Pulay forces ***
C
      CALL PULAY(GRADIENT,PA,NSTO,DDEG/3)
C
C     *** Build SCF density matrix ***
C
      CALL BLDPMAT(CA,PA,NSTO,OCCNA,LFOMOA,HFOMOA)
      IF (OKS) CALL BLDPMAT(CB,PB,NSTO,OCCNB,LFOMOB,HFOMOB)
C
  100 CONTINUE
C
C     *** Calculate Fock exchange energy forces ***
C
      IF (OKS) THEN
        CALL D1FOCK(GRADIENT,PA,PB,CA,CB,NSTO,NSTO_ME,DDEG/3)
      ELSE
        CALL D1FOCK(GRADIENT,PA,PA,CA,CA,NSTO,NSTO_ME,DDEG/3)
      END IF
C
C     *** Calculate exchange-correlation energy forces ***
C
      CALL BLDCDC(NCDCA,'BUILD ALPHA')
      IF (OKS) THEN
        CALL BLDCDC(NCDCB,'BUILD BETA')
        CALL D1NUMXC(GRADIENT,PA,PB,NCDCA,NCDCB,NSTO,DDEG/3)
      ELSE
        CALL D1NUMXC(GRADIENT,PA,PA,NCDCA,NCDCB,NSTO,DDEG/3)
      END IF
C
C     *** Calculate Coulomb energy forces ***
C
      CALL D1BLDCDC(NCDCA,NCDCB,'D1COULOMB')
      IF (CDXC.AND.OKS) THEN
        CALL D1COULOMB(GRADIENT,PA,PB,NCDCA,NCDCB,NSTO,DDEG/3)
      ELSE IF (OKS) THEN
        PA(:,:) = PA(:,:) + PB(:,:)
        CALL D1COULOMB(GRADIENT,PA,PB,NCDCA,NCDCB,NSTO,DDEG/3)
        PA(:,:) = PA(:,:) - PB(:,:)
      ELSE
        CALL D1COULOMB(GRADIENT,PA,PA,NCDCA,NCDCB,NSTO,DDEG/3)
      END IF
C
C     *** Build total density matrix ***
C
      IF (OKS) PA(:,:) = PA(:,:) + PB(:,:)
C
C     *** Calculate fixed Coulomb energy forces ***
C
      IF (FITTYP.EQ.'VALENCE') THEN
        CALL D1BLDCDC(NCDCA,NCDCB,'D1COULOMB_FIX')
        CALL D1COULOMB_FIX(GRADIENT,PA,NCDCA,NSTO,DDEG/3)
      END IF
C
C     *** Calculate kinetic energy forces ***
C
      CALL D1KINETIC(GRADIENT,PA,NSTO,DDEG/3)
C
C     *** Calculate core energy forces ***
C
      CALL D1CORE(GRADIENT,PA,NSTO,DDEG/3)
C
C     *** Calculate ECP forces ***
C
      IF (ECPS) CALL D1ECPCORE(GRADIENT,PA,NSTO,DDEG/3)
C
C     *** Calculate MCP forces ***
C
      IF (MCPS) CALL D1MCPCORE(GRADIENT,PA,NSTO,DDEG/3)
C
C     *** Calculate embedding energy forces ***
C
      IF (EMBED) CALL D1EMBEDDING(GRADIENT,PA,NSTO,DDEG/3)
C
C     *** Calculate MM embedding energy forces ***
C
      IF (QMMM) CALL D1EMBEDMM(GRADIENT,PA,NSTO,DDEG/3)
C
C     *** Collect gradient parts ***
C
      CALL MP_COLLECT(GRADIENT,DDEG,MASTER,(/1.0/),.FALSE.)
C
      IF (MPSLAVE) GO TO 200
C
C     *** Calculate Dunlap forces ***
C
      CALL D1BLDCDC(NCDCA,NCDCB,'D1DUNLAP')
      CALL D1DUNLAP(GRADIENT,NCDCA,NCDCB,DDEG/3)
C
C     *** Calculate fixed Dunlap forces ***
C
      IF (FITTYP.EQ.'VALENCE') THEN 
        CALL D1BLDCDC(NCDCA,NCDCB,'D1DUNLAP_FIX')
        CALL D1DUNLAP_FIX(GRADIENT,NCDCA,NCDCB,DDEG/3)
      END IF
C
C     *** Calculate force contributions from the external field ***
C
      IF (EFIELD) THEN
        CALL MODEFIELD (EFMOD1,EFMOD2,RTT,EFCT,EFWT,EFPUL)
        DO IFIELD=1,3
          FIELD = ELFIELD(IFIELD)
          FIELD = FIELD*EFMOD1*EFMOD2
          CALL D1NUCDIP(GRADIENT,FIELD,IFIELD,0,DDEG/3)
          CALL D1DIPOLE(GRADIENT,PA,FIELD,IFIELD,0,NSTO,DDEG/3)
        END DO
      END IF
C
  200 CONTINUE
C
C     *** Deallocate local fields ***
C
      IF (OKS) THEN
        DEALLOCATE(CA,PA,CB,PB,STAT=ALLOCATION)
      ELSE
        DEALLOCATE(CA,PA,STAT=ALLOCATION)
      END IF
      CALL CHKERRSTAT(ALLOCATION,'QMGRAD','DEALLOCATION FAILED')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE QMGRAD ***
C
      END
