      SUBROUTINE TRFMOAMAGMA(hMATRIX,DSTO,DSTO_ME,OPTION)
C
C     Purpose: Perform AO <-> MO transformations with MAGMA.
C
C     History: - Creation (10.08.21, KH)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DSTO: Dimension of MATRIX.
C
C     List of local variables:
C    
C     MATRIX : Matrix to be transformed -> The transfored matrix MO(OA)  
C     OPTION : a) PAO->PMO, transform density matrix from AO to MO basis
C              b) PMO->PAO, transform density matrix from MO to AO basis
C              c) KAO->KMO, transform Fock matrix from AO to MO basis
C     S      : Overlap matrix -> Canonical orthogonalization matrix.
C     Y      : Y = S*eigenvalue
C     C      : Work matrix. 
C
C     ------------------------------------------------------------------
C
      use iso_c_binding
      use magma
      IMPLICIT NONE
C     
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'molecule.h'
      INCLUDE 'rtdft.h' 
      INCLUDE 'tolerance.h'
      INCLUDE 'slp.h'
C
      CHARACTER*(*) OPTION
      INTEGER ALLOCATION,I
      INTEGER(kind=4) DSTO,DSTO_ME
      REAL hMATRIX(DSTO_ME,DSTO_ME)

C     square of DTSO
      integer(kind=4) :: DSTO_square
C
C     factor for the magmablasf_dgeadd subroutine such as
C     dB = alpha * dA + dB and
      real(kind=8) :: alpha_prod
      real(kind=8) :: beta_prod

C     exit error info 
      integer(kind=4)               :: info
C
      real(kind=8),allocatable      :: hS(:,:)
      real(kind=8),allocatable      :: hY(:,:)
      integer(kind=8)  :: dMATRIX
      integer(kind=8)  :: dS
      integer(kind=8)  :: dY
      integer(kind=8)  :: dC

C     setting the variables
      DSTO_square = DSTO * DSTO
      alpha_prod = 1.
      beta_prod = 0.

C
C     *** Get transformation and eigenvalue eigenvalue matrices ***
C
      allocate(hS(DSTO,DSTO))
      hS(:,:) = 0.0
      call SCFIO(hS,DSTO_ME,'READ','ORTHO')
C
C     ------------------------------------------------------------------
C
C     *** Allocation ***
C
      info = magmaf_dmalloc( dMATRIX, DSTO_square )
      info = magmaf_dmalloc( dC, DSTO_square )

C     The MATRIX is sent from the host to the GPU device
      call magmaf_dsetmatrix(DSTO, DSTO, hMATRIX, DSTO,
     $ dMATRIX, DSTO, magma_queue)

C
C     *** Transformation PAO -> PMO ***
C
      IF (OPTION.EQ.'PAO->PMO') THEN
 
        info = magmaf_dmalloc( dY, DSTO_square )
 
        allocate(hY(DSTO,DSTO))
        hY(:,:) = 0.0
        DO I=1,NORTH
          hY(1:DSTO_ME,I) = hS(1:DSTO_ME,I)*EGIS(I)
        END DO

C     The Y matrix is sent from the host to the GPU device
        call magmaf_dsetmatrix(DSTO, DSTO, hY, DSTO,
     $       dY, DSTO, magma_queue)
      
        call magmablasf_dgemm(MagmaTrans, MagmaNoTrans,
     $       DSTO, DSTO, DSTO, alpha_prod, dY, DSTO, dMATRIX,
     $       DSTO, beta_prod, dC, DSTO, magma_queue)

        call magmablasf_dgemm(MagmaNoTrans, MagmaNoTrans,
     $       DSTO, DSTO, DSTO, alpha_prod, dC, DSTO, dY,
     $       DSTO, beta_prod, dMATRIX, DSTO, magma_queue)

        deallocate(hY,STAT=ALLOCATION)
        info = magmaf_free(dY)
C
C     *** Transformation PMO -> PAO ***
C
      ELSE IF (OPTION.EQ.'PMO->PAO') THEN
C
        info = magmaf_dmalloc( dS, DSTO_square )

C     The S matrix is sent from the host to the GPU device
        call magmaf_dsetmatrix(DSTO, DSTO, hS, DSTO,
     $       dS, DSTO, magma_queue)

        call magmablasf_dgemm(MagmaNoTrans, MagmaNoTrans,
     $       DSTO, DSTO, DSTO, alpha_prod, dS, DSTO, dMATRIX,
     $       DSTO, beta_prod, dC, DSTO, magma_queue)

        call magmablasf_dgemm(MagmaNoTrans, MagmaTrans,
     $       DSTO, DSTO, DSTO, alpha_prod, dC, DSTO, dS,
     $       DSTO, beta_prod, dMATRIX, DSTO, magma_queue)

        info = magmaf_free(dS)
C  
C     *** Transformation KAO -> KMO ***
C
      ELSE IF (OPTION.EQ.'KAO->KMO') THEN
C
        info = magmaf_dmalloc( dS, DSTO_square )

C     The S matrix is sent from the host to the GPU device
        call magmaf_dsetmatrix(DSTO, DSTO, hS, DSTO,
     $       dS, DSTO, magma_queue)

        call magmablasf_dgemm(MagmaTrans, MagmaNoTrans,
     $       DSTO, DSTO, DSTO, alpha_prod, dS, DSTO, dMATRIX,
     $       DSTO, beta_prod, dC, DSTO, magma_queue)

        call magmablasf_dgemm(MagmaNoTrans, MagmaNoTrans,
     $       DSTO, DSTO, DSTO, alpha_prod, dC, DSTO, dS,
     $       DSTO, beta_prod, dMATRIX, DSTO, magma_queue)

        info = magmaf_free(dS)
C
      ELSE
C
        CALL ERRMSG('TRFMOA','UNKNOWN OPTION',1)
C
      END IF

C     The MATRIX is sent back from the GPU device
C     to the host
      call magmaf_dgetmatrix(DSTO, DSTO,
     $     dMATRIX, DSTO, hMATRIX, DSTO, magma_queue)

C
C     *** Deallocation ***
C
      deallocate(hS,STAT=ALLOCATION)
      info = magmaf_free(dMATRIX)
      info = magmaf_free(dC)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE TRFMOAMAGMA ***
C
      END

