      SUBROUTINE PLOTIMF
C
C     Purpose: Driver routine for (parallel) induced magnetic field
C              plot function calculation.
C
C     History: - Creation (14.06.13, BZG)
C                         (06.09.13, AMK)
C
C     ******************************************************************
C
C     List of local variables:
C
C     NBATCH: Number of batches.
C     NGPB  : Number of grid points in current batch.
C     NGPLB : Number of grid points in last batch.
C     NICUBE: Number of cubes along i lattice vector.
C     NJCUBE: Number of cubes along j lattice vector.
C     NKCUBE: Number of cubes along k lattice vector.
C     
C     List of local dynamical fields:
C
C     FUNVEC: Vector with plot function values.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'fileio.h'
      INCLUDE 'flags.h'
      INCLUDE 'grid.h'
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'plot.h'
C
      LOGICAL SUCCESS
      INTEGER I,IBATCH,ICUBE,IGP,JCUBE,KCUBE
      INTEGER NBATCH,NGPB,NGPLB,NICUBE,NJCUBE,NKCUBE,TAPE
C
      REAL VECTOR(3)
C
      INTEGER ALLOCATION
      REAL,ALLOCATABLE :: FUNVEC(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Set plot function names ***
C
      CALL PIEDRV('PIENAME')
C
C     *** Build pertubed GIAO density matrix ***
C
      CALL BLDPGIAO
C
C     *** Allocate vector for plot function values ***
C
      ALLOCATE (FUNVEC(MAXGPB,MAXPLOT),STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PLOTIMF','ALLOCATION FAILED',1)
      END IF
C
C     *** Plot function on structured lattice ***
C
      IF (LATTYP.EQ.'STRUCTURED') THEN
C
        CALL STORAD(1.0E-10)
        CALL DIVIDE(NICUBE,NJCUBE,NKCUBE,'STEPS')
C
C     *** Loop over structured grid ***
C
        DO KCUBE=1,NKCUBE
          DO JCUBE=1,NJCUBE
            DO ICUBE=1,NICUBE
              CALL CUBUS(ICUBE,JCUBE,KCUBE,NGPB,'STEPS')
              CALL VICINITY(ICUBE,JCUBE,KCUBE,GRDVEC(:,1:),'STEPS')
              CALL IMF(FUNVEC,NGPB)
              IF (MPMASTER) CALL CUBEIO(FUNVEC,ICUBE,JCUBE,KCUBE)
            END DO
          END DO
        END DO
C
C     *** Plot function on unstructured lattice ***
C
      ELSE IF (LATTYP.EQ.'POINTS') THEN
C
        CALL STORAD(1.0E-10)
C
        IF (LATINP.EQ.'INTERNAL') THEN
          CALL FINDKEY('POINTS',SUCCESS)
          IF (.NOT.SUCCESS) GO TO 99980
          TAPE = INP
        ELSE IF (LATINP.EQ.'EXTERNAL ASCII') THEN
          REWIND (LAT)
          READ (LAT,*,ERR=99980,END=99990) NLATTICE(0)
          TAPE = LAT
        ELSE IF (LATINP.EQ.'EXTERNAL BINARY') THEN
          RECNO(LAT) = 1
          TAPE = LAT
        END IF
C
        NBATCH = NLATTICE(0)/MAXGPB
        NGPLB = MOD(NLATTICE(0),MAXGPB)
        IF (NGPLB.NE.0) NBATCH = NBATCH + 1
C
C     *** Loop over unstructured lattice ***
C
        DO IBATCH=1,NBATCH
          IF (IBATCH.EQ.NBATCH.AND.NGPLB.NE.0) THEN
            NGPB = NGPLB
          ELSE
            NGPB = MAXGPB
          END IF
          DO IGP=1,NGPB
            IF (LATINP.EQ.'EXTERNAL BINARY') THEN
              RECNO(LAT) = RECNO(LAT) + 1
              CALL BINARY(VECTOR,3,3,RECNO(LAT),LAT,'LAT.bin','READ')
            ELSE
              READ (TAPE,*,ERR=99980,END=99990) (VECTOR(I),I=1,3)
            END IF
            CALL MODPOINT(VECTOR)
            GRDVEC(1:3,IGP) = VECTOR(1:3)/ANGBOR(1)
          END DO
          CALL VICINITY(NGPB,0,0,GRDVEC(:,1:),'POINTS')
          CALL IMF(FUNVEC,NGPB)
          IF (MPMASTER) CALL UNSTIO(FUNVEC,IBATCH,NGPB)
        END DO
      END IF
C
C     *** Convert binary data to ascii ***
C
      IF ((PLOT.EQ.'ASCII').OR.(PLOT.EQ.'TABLE')) THEN
        IF (MPMASTER) CALL BIN2ASC
      END IF
C
CCC ADLL 
C
C     *** Convert binary data to cube ***
C
      IF (PLOT.EQ.'CUBE') THEN
        CALL BIN2CUBE
      END IF
CCC ADLL 
C
C     *** Deallocate vector for plot function values ***
C
      DEALLOCATE(FUNVEC,STAT=ALLOCATION)
      IF (ALLOCATION.GT.0) THEN
        CALL ERRMSG('PLOTIMF','DEALLOCATION FAILED',1)
      END IF
C
      RETURN
C
C     ------------------------------------------------------------------
C
C     *** Handling of errors ***
C
99980 CONTINUE
      CALL ERRMSG('PLOTIMF','I/O ERROR READING UNSTRUCTURED LATTICE',1)
C
99990 CONTINUE
      CALL ERRMSG('PLOTIMF','EOF WHILE READING UNSTRUCTURED LATTICE',1)
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE PLOTIMF ***
C
      END
