
C###############################################################################
C   subroutine / function  mmbsjn
C
C   imsl routine name   - mmbsjn
C
C-----------------------------------------------------------------------
C
C   computer            - cray/single
C
C   latest revision     - november 1, 1984
C
C   purpose             - bessel function of the first kind of
C                           nonnegative integer order for
C                           real arguments
C
C   usage               - call mmbsjn (arg,n,b,ier)
C
C   arguments    arg    - input argument. the absolute value of arg must
C                           be less than or equal to 100000. arg must be
C                           typed appropriately in the calling program.
C                           (see the precision/hardware section.)
C                n      - input parameter specifying the number of
C                           function values to be computed.
C                b      - output vector of length n containing the
C                           computed function values. b must be typed
C                           appropriately in the calling program.
C                           (see the precision/hardware section.)
C                           b(1) will contain the computed value for
C                           order zero, b(2) will contain the computed
C                           value for order 1, b(3) for order 2, etc.
C                ier    - error parameter. (output)
C                         terminal error
C                           ier = 129 indicates that either arg or n is
C                             out of range. b(i), (i=1,n) is set to
C                             machine infinity.
C                           ier = 129 + j indicates that b(i), (i=1,j)
C                             are computed to machine precision, but
C                             precision is lost for b(i), (i=j+1,n.)
C                             see the programming notes.
C
C   precision/hardware  - double/h32,h36
C                       - single/h48,h60
C
C   reqd. imsl routines - uertst,ugetio
C
C   notation            - information on special notation and
C                           conventions is available in the manual
C                           introduction or through imsl routine uhelp
C
C   copyright           - 1982 by imsl, inc. all rights reserved.
C
C   warranty            - imsl warrants only that imsl testing has been
C                           applied to this code. no other warranty,
C                           expressed or implied, is applicable.
C
C-----------------------------------------------------------------------
C
      SUBROUTINE chebybess (arg,n,b,ier)
      IMPLICIT NONE
C                                  specifications for arguments
      integer            n,ier
      real             arg,b(*)
C                                  specifications for local variables
      integer            l,largex,magx,ncalc,nn,nbmx,m,nstart,nend
      integer            itemp
      real  test,tempa,tempb,tempc,p
      real  rsign,sum,tover,plast,pold,psave,psavel
      real  dsig,rten,tmpa4,smallx,xinf,ONETENTH, ONE,TWO,SEVEN
      real  HALF,FOUR,FOURTH,TEN,ZERO
      data               dsig/7.0d0/
      data               rten/44.d0/
      data               xinf/1.7d+38/
      data               largex /100000/
C                                  first executable statement
      ONETENTH=0.10
      HALF=0.5
      ZERO=0.0
      ONE=1.0
      TWO=2.0
      FOUR=4.0
      FOURTH=0.25
      TEN=10.0
      SEVEN=7.0
      ier = 0
      tempa = abs(arg)
      magx =  tempa
      if(n.gt.0 .and. magx.le.largex) go to 10
C                                  error return -- arg,n is out of range
      ier = 129
      b(1) = xinf
      if(n.lt.2) go to 9000
      do 5 l=2,n
         b(l) = xinf
    5 continue
      go to 9000
   10 rsign = ONE
      ncalc = n
C                                  use 2-term ascending series for
C                                    small arg
      tmpa4 = tempa**FOUR
      smallx = ONETENTH**dsig
      if(tmpa4.ge.smallx) go to 20
C                                  two-term ascending series for
C                                    small arg
      tempa = ONE
      tempb = -FOURTH*arg*arg*rsign
      b(1) = ONE+tempb
      if(n.eq.1) go to 9005
      do 15 nn=2,n
         tempa = tempa*arg/real(2*nn-2)
         b(nn) = tempa*(ONE+tempb/real(nn))
   15 continue
      go to 9005
C                                  initialize the calculation of p*s
   20 nbmx = n-magx
      nn = magx+1
      plast = ONE
      p = real(2*nn)/tempa
C                                  calculate general significance test
      test = TWO * TEN**dsig
      m = 0
      if(nbmx.lt.3) go to 30
C                                  calculate p*s until nn=n-1.
C                                    check for possible overflow.
      tover = TEN**(rten-dsig)
      nstart = magx+2
      nend = n-1
      do 25 nn=nstart,nend
         pold = plast
         plast = p
         p = real(2*nn)*plast/tempa-rsign*pold
         if(p-tover) 25, 25, 35
   25 continue
      nn = nend
C                                  calculate special significance test
C                                    for nbmx.gt.2.
C
      test = max(test,sqrt(plast*TEN**dsig)*sqrt(TWO*p))
C
C                                  calculate p*s until significance
C                                    test passes
   30 nn = nn+1
      pold = plast
      plast = p
      p = real(2*nn)*plast/tempa-rsign*pold
      if(p.lt.test) go to 30
      if(m.eq.1) go to 55
C                                  for j*s, a strong variant of the test
C                                    is necessary. calculate it, and
C                                    calculate p*s until this test is
C                                    passed.
      m = 1
      tempb = p/plast
      tempc = real(nn+1)/tempa
      if(tempb+1./tempb.gt.2.*tempc)tempb=tempc+sqrt(tempc**2-ONE)
      test = test/sqrt(tempb-ONE/tempb)
      if(p-test) 30, 55, 55
C                                  to avoid overflow, divide p*s by
C                                    tover.  calculate p*s until
C                                    abs(p).gt.1.
   35 tover = TEN**rten
      p = p/tover
      plast = plast/tover
      psave = p
      psavel = plast
      nstart = nn+1
   40 nn = nn+1
      pold = plast
      plast = p
      p = real(2*nn)*plast/tempa-rsign*pold
      if(p.le.1.) go to 40
      tempb = real(2*nn)/tempa
      tempc = HALF*tempb
      tempb = plast/pold
      if(tempb+ONE/tempb .gt. TWO*tempc)                                
     $                  tempb=tempc+sqrt(tempc**2-ONE)
C
C                                  calculate backward test, and find
C                                    ncalc, the highest nn such that the
C                                    test is passed.
      test = HALF*pold*plast*(ONE-ONE/tempb**2)/TEN**dsig
      p = plast*tover
      nn = nn-1
      nend = min(n,nn)
      do 45 ncalc=nstart,nend
         pold = psavel
         psavel = psave
         psave = real(2*nn)*psavel/tempa-rsign*pold
         if(psave*psavel-test) 45, 45, 50
   45 continue
      ncalc = nend+1
   50 ncalc = ncalc-1
C                                  the sum b(1)+2b(3)+2b(5)... is used
C                                    to normalize. m, the coefficient of
C                                    b(nn), is initialized to 2 or 0.
   55 nn = nn+1
      m = 2*nn-4*(nn/2)
C                                  initialize the backward recursion and
C                                    the normalization sum
      tempb = ZERO
      tempa = ONE/p
      sum = real(m)*tempa
      nend = nn-n
      if(nend) 80, 70, 60
C                                  recur backward via difference
C                                    equation, calculating (but not
C                                    storing) b(nn), until nn=n.
   60 do 65 l=1,nend
         nn = nn-1
         tempc = tempb
         tempb = tempa
         tempa = real(2*nn)*tempb/arg-rsign*tempc
         m = 2-m
         sum = sum+real(m)*tempa
   65 continue
C                                  store b(nn)
   70 b(nn) = tempa
      if(n.gt.1) go to 75
C                                  n=1.  since 2*tempa is added to the
C                                    sum, tempa must be subtracted
      sum = sum-tempa
      go to 110
C                                  calculate and store b(nn-1)
   75 nn = nn-1
      b(nn) = real(2*nn)*tempa/arg-rsign*tempb
      if(nn.eq.1) go to 105
      m = 2-m
      sum = sum+real(m)*b(nn)
      go to 90
C                                  nn.lt.n, so store b(nn) and set
C                                  higher orders to zero
   80 b(nn) = tempa
      nend = -nend
      do 85 l=1,nend
         itemp = nn+l
         b(itemp) = ZERO
   85 continue
   90 nend = nn-2
      if(nend.eq.0) go to 100
C                                  calculate via difference equation and
C                                    store b(nn), until nn=2
      do 95 l=1,nend
         nn = nn-1
         b(nn) = real(2*nn)*b(nn+1)/arg-rsign*b(nn+2)
         m = 2-m
         sum = sum+real(m)*b(nn)
   95 continue
C                                  calculate b(1)
  100 b(1) = TWO*b(2)/arg-rsign*b(3)
  105 sum = sum+b(1)
C                                  normalize--if ize=1, divide sum by
C                                    cosh(arg). divide all b(nn) by sum.
  110 continue
      do 115 nn=1,n
  115 b(nn) = b(nn)/sum
      if(ncalc.eq.n) go to 9005
      ier = 129+ncalc
 9000 continue
C      call uertst(ier,'mmbsjn')
 9005 return
      end 

