      SUBROUTINE NEAREMBEDRAD(H,DSTO)
C
C     Purpose: Calculation of near-field embedding nuclear attraction
C              type integrals (NAIs).
C
C     Lit.: C. Quintanar et al., IJQC 96, 483 (2004)
C
C     History: - Creation (20.08.12, AAI)
C                         (03.03.13, GG)
C                         (18.02.15, AMK)
C
C     ******************************************************************
C
C     List of local dimensions:
C
C     DCOP: Dimension of Cartesian orbitals.
C     DPOP: Dimension of product orbitals.
C     DSHL: Dimension of shell blocks.
C     DSPP: Dimension of scaling parameters.
C     DSTO: Dimension of Slater typ orbital matrix.
C
C     List of local variables:
C
C     H        : Core Hamiltonian -> Embedded core Hamiltonian.
C     L{A/B}   : Angular quantum number of orbitals.
C     NGTO{A/B}: Number of contractions.
C     SKIPPAIR : Skipping of atom pair, if true.
C     SKIPSHL  : Skipping of shell combination, if true.
C
C     List of local dynamical fields:
C
C     NEARAP: Near-field relation between atom A and point.
C     NEARBP: Near-field relation between atom B and point.
C     ERIS  : Sum of one center electron repulsion integrals.
C     HRRINT: Work field for horizontal recurrence relation.
C     ISF   : Integral scaling factors.
C     PHG   : Primitive Hermite Gaussian integrals.
C     SHLBLK: Diatomic integral shell block.
C     TRRINT: Work field for transformation recurrence relation.
C     VRRINT: Work field for vertical recurrence relation.
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'molecule.h'
      INCLUDE 'mp_demon.h'
      INCLUDE 'physic.h'
      INCLUDE 'pointer.h'
      INCLUDE 'projectile.h'
C
      INTEGER DSTO
      REAL H(DSTO,DSTO)
C
      LOGICAL SKIPPAIR,SKIPSHL
      INTEGER IATOM,ISHL,JATOM,JSHL,KRAD,LA,LB,NGTOA,NGTOB
C
      INTEGER LLSHL_ME,NSHL_ALL,ULSHL_ME
      REAL TGLOBAL,TLOCAL
      REAL FACTOR,BETA,CLAU,MC2,VANGAC,VANGBC,ANGLE
C
      INTEGER ALLOCATION,DCOP,DIMCOP,DIMPOP,DIMSOP,DPOP,DSHL,DSPP
      LOGICAL,ALLOCATABLE :: NEARAP(:),NEARBP(:)
      REAL,ALLOCATABLE :: ERIS(:,:,:),HRRINT(:,:),ISF(:,:,:),
     $                    PHG(:,:,:),SHLBLK(:,:),TRRINT(:,:,:),
     $                    VRRINT(:,:)
C
C     ------------------------------------------------------------------
C
C     *** Allocate local fields for near-field embedding integrals ***
C
      DCOP = DIMCOP(2*LBASMAX(0))
      DPOP = DIMPOP(2*LBASMAX(0))
      DSHL = DIMSOP(LBASMAX(0))
      DSPP = DIMSOP(2*LBASMAX(0))
C
      ALLOCATE(NEARAP(NRADP),NEARBP(NRADP),ERIS(MAXCON,MAXCON,DCOP),
     $         HRRINT(DCOP,DCOP),SHLBLK(DSHL,DSHL),TRRINT(DPOP,DSPP,2),
     $         ISF(MAXCON,MAXCON,DSPP),PHG(MAXCON,MAXCON,DCOP),
     $         VRRINT(DCOP,0:MAXLUL),STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'NEAREMBEDRAD','ALLOCATION FAILED')
C
C     *** Initialize integral recurrence fields ***
C
      MC2 = MRAD*AMU*(CLIGHT/(ABOHR*FSEC*1.E15))**2
      CLAU = CLIGHT/(ABOHR*FSEC*1.E15)
      BETA = SQRT(1 - (KINRAD/MC2 + 1)**(-2))
C
      HRRINT(:,:) = 0.0
      SHLBLK(:,:) = 0.0
      VRRINT(:,:) = 0.0
C
C     *** Calculate parallel loop loads ***
C
      IF (MPINI(MP_EMBN)) THEN
        CALL MYERILOAD(H,TGLOBAL,TLOCAL,LLSHL_ME,ULSHL_ME,MP_EMBN,
     $                 'INITIALIZE')
      ELSE
        CALL MYERILOAD(H,TGLOBAL,TLOCAL,LLSHL_ME,ULSHL_ME,MP_EMBN,
     $                 'DISTRIBUTE')
      END IF
C
C     *** Calculate embedding integrals and add to H ***
C
      NSHL_ALL = 0
      CALL MPCPUTIM(TLOCAL,'START')
C
      DO 50 IATOM=1,NATOM
        CALL NEARFIELD(IATOM,NEARAP,NRADP,'RADIATION')
        DO 40 JATOM=IATOM,NATOM
          CALL ATOMDIS(IATOM,JATOM,RAB,RSQ)
          IF (SKIPPAIR(IATOM,JATOM,RSQ)) GO TO 40
          CALL NEARFIELD(JATOM,NEARBP,NRADP,'RADIATION')
          DO 30 ISHL=LLSHL(IATOM),ULSHL(IATOM)
            DO 20 JSHL=MAX(ISHL,LLSHL(JATOM)),ULSHL(JATOM)
              LA = SHLPTR(ISHL,3)
              LB = SHLPTR(JSHL,3)
              CALL VSETUP(LA,LB,NGTOA,NGTOB,ISHL,JSHL)
              CALL VSCALE(LA,LB,-1,NGTOA,NGTOB,ISF,DSPP)
              CALL GTOPRO(NGTOA,NGTOB,SKIPSHL)
              IF (SKIPSHL) GO TO 20
              NSHL_ALL = NSHL_ALL + 1
              IF ((NSHL_ALL.LT.LLSHL_ME).OR.
     $            (NSHL_ALL.GT.ULSHL_ME)) GO TO 20
              ERIS(:,:,:) = 0.0
              DO 10 KRAD=1,NRADP
                IF ((.NOT.NEARAP(KRAD)).AND.
     $              (.NOT.NEARBP(KRAD))) GO TO 10
                CALL RADVEC(IATOM,KRAD,RAC)
                IF (USELWP) THEN
                  VANGAC = ANGLE(RAC(1),RAC(2),RAC(3),
     $                     VIORAD(1,KRAD),VIORAD(2,KRAD),VIORAD(3,KRAD))
                  FACTOR = (1.0 - ((BETA*SIN(VANGAC))**2))**0.5
                ELSE
                  FACTOR = 1.0
                END IF

                CALL RADVEC(JATOM,KRAD,RBC)
                CALL RVRR(LA,LB,0,NGTOA,NGTOB,PHG,VRRINT,DCOP)
                CALL CUBSUM(LA,LB,QRAD(KRAD)/FACTOR,NGTOA,NGTOB,ERIS,
     $                      PHG,DCOP)
   10         CONTINUE
              CALL VCONT(LA,LB,NGTOA,NGTOB,ERIS,ISF,TRRINT,
     $                   DCOP,DPOP,DSPP)
              CALL TRR(LA,LB,HRRINT,TRRINT,DCOP,DPOP,DSPP)
              CALL HRR(LA,LB,HRRINT,SHLBLK,DCOP,DSHL,'TRANS')
              CALL BLDIMAT(LA,LB,ISHL,JSHL,LLSTO,NCSTO,H,SHLBLK,
     $                     NSTO,DSHL)
   20       CONTINUE
   30     CONTINUE
   40   CONTINUE
   50 CONTINUE
C
C     *** Update parallel loop loads ***
C
      CALL MYERILOAD(H,TGLOBAL,TLOCAL,LLSHL_ME,ULSHL_ME,MP_EMBN,
     $               'UPDATE')
C
C     *** Deallocate local fields for near-field embedding integrals ***
C
      DEALLOCATE(NEARAP,NEARBP,ERIS,HRRINT,ISF,PHG,SHLBLK,TRRINT,VRRINT,
     $           STAT=ALLOCATION)
      CALL CHKERRSTAT(ALLOCATION,'NEAREMBEDRAD','DEALLOCATION FAILED')
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE NEAREMBED ***
C
      END
