      SUBROUTINE XCPOT(RHOALP,RHOBET,RHOTOT,DRHOALP,DRHOBET,DRHOAB,
     $                 DRHOTOT,KINALP,KINBET,ZSPIN,VXALP,VXBET,VXGALP,
     $                 VXGBET,VXTALP,VXTBET,VCALP,VCBET,VCGALP,VCGBET,
     $                 VCG2ALP,VCG2BET,VCTALP,VCTBET,NGPB)
C
C     Purpose: Calculate exchange-correlation potential.
C
C     History: - Creation (10.10.00, AMK)
C                         (27.10.00, AVA)
C                         (01.04.18, FAD)
C
C     ******************************************************************
C
C     List of local variables:
C
C     DRHO{ALP/BET}: Norm of the gradient of the alpha/beta density.
C     DRHOAB       : Product of alpha and beta density gradient.
C     DRHOTOT      : Norm of the gradient of the total density.
C     KIN{ALP/BET} : Kinetic energy density for alpha/beta spin.
C     NGPB         : Number of grid points in the current batch.
C     RHO{ALP/BET} : Alpha/Beta density.
C     RHOTOT       : Total electron density.
C     VC{ALP/BET}  : Correlation potential of the alpha/beta density.
C     VCG{ALP/BET} : GGA correction to the correlation potential.
C     VCG2{ALP/BET}: GGA (LYP) correction to the correlation potential.
C     VCT{ALP/BET/}: Meta-GGA correction to the correlation potential.
C     VX{ALP/BET}  : Exchange potential of the alpha/beta density.
C     VXG{ALP/BET} : GGA correction to the exchange potential.
C     VXT{ALP/BET} : Meta-GGA correction to the exchange potential.
C     ZSPIN        : Relative spin polarization.
C
C     List of exchange potential subroutines:
C
C     VXALPHA  : X-alpha (Dirac) exchange potential.
C     VXLSD    : Uniform electron gas exchange potential.
C     VXPW86   : Perdew-Wang GGA exchange potential (1986).
C     VXB88    : Becke GGA exchange potential (1988).
C     VXPW91   : Perdew-Wang GGA exchange potential (1991).
C     VXECMV   : Engel et al. GGA exchange potential (1992).
C     VXEV93   : Engel & Vosko GGA exchange potential (1993).
C     VXLB94   : Leeuwen-Baereends model potential (1994).
C     VXPBE96  : Perdew et al. GGA exchange potential (1996).
C     VXPBE98  : Zhang & Yang PBE exchange potential (1998).
C     VXPPB99  : Hammer, Hansen & Norskov PBE exchange potential (1999).
C     VXPBESOL : PBEsol GGA exchange potential (2008).
C     VXHC     : Handy & Cohen GGA exchange potential (2001).
C     VXKT1    : Keal & Tozer GGA exchange potential 1 (2003).
C     VXKT2    : Keal & Tozer GGA exchange potential 2 (2003).
C     VXKT3    : Keal & Tozer GGA exchange potential 3 (2004).
C     VXVMT    : Vela, Medel & Trickey GGA exchange potential (2009).
C     VXVMTSOL : VMTsol exchange potential (2009).
C     VXSO11   : Peverati, Zhao & Truhlar GGA exchange potential (2011).
C     VXN12    : Peverati & Truhlar GGA exchange potential (2012). 
C     VXGAM    : Yu et al. GGA type exchange potential (2015).
C     VXCAP    : Carmona-Espindola et al. GGA exchange potential (2014).
C     VXVS98   : Van Voohris & Scuseria mGGA exchange potential (1998).
C     VXPKZB   : Perdew et al. mGGA exchange potential (1999).
C     VXTPSS   : Tao et al. mGGA exchange potential (2003).
C     VXM06    : Zhao & Truhlar mGGA exchange potential (2006).
C     VXM11    : Peverati & Trular mGGA exchange potential (2011).
C     VXMN12   : Peverati & Trular mGGA exchange potential (2012).
C     VXSCAN   : Sun et al. mGGA exchange potential (2015).
C     VXB3LYP  : B3LYP hybrid exchange potential DFT part (1994).
C     VXPBE0   : PBE0 hybrid exchange potential DFT part (1996).
C     VXCAMB88 : CAMB3LYP RS hybrid exchange potential DFT part (2004).
C     VXCAMPBE : CAMPBE0 RS hybrid exchange potential DFT part (2004).
C     VXRCAMB88: Revised CAMB3LYP RS hybrid exchange potential part.
C
C     List of correlation potential subroutines:
C
C     VCPVWN  : Pade interpolated VWN correlation potential (1980).
C     VCRVWN  : RPA interpolated VWN correlation potential (1980).
C     VCPW92  : Perdew & Wang local correlation potential (1992).
C     VCP86   : Perdew correlation potential (1986).
C     VCLYP   : Lee-Yang-Parr correlation potential (1989).
C     VCPW91  : Perdew & Wang GGA correlation potential (1991).
C     VCPBE   : Perdew, Burke & Ernzerhof correlation potential (1996).
C     VCSPBE  : Spin stabilized PBE correlation potential (2020).
C     VCPBESOL: PBEsol correlation potential (2008).
C     VCKT2   : Keal & Tozer local correlation potential (2003).
C     VCKT3   : Keal & Tozer GGA correlation potential (2004).
C     VCSO11  : Peverati et al. GGA correlation potential (2011).
C     VCN12   : Peverati & Truhlar GGA correlation potential (2012).
C     VCGAM   : Yu et al. GGA type correlation potential (2015).
C     VCVS98  : Van Voohris et al. mGGA correlation potential (1998).
C     VCPKZB  : Perdew et al. mGGA correlation potential (1999).
C     VCTPSS  : Tao et al. mGGA correlation potential (2003).
C     VCM06   : Zhao & Truhlar mGGA correlation potential (2006).
C     VCM11   : Peverati & Trular mGGA correlation potential (2011).
C     VCMN12  : Peverati & Truhlar mGGA correlation potential (2012).
C     VCSCAN  : Sun et al. mGGA correlation potential (2015).
C     VCB3LYP : B3LYP hybrid correlation potential (1994).
C
C     ------------------------------------------------------------------
C
      IMPLICIT NONE
C
      INCLUDE 'parameter.h'
C
      INCLUDE 'flags.h'
      INCLUDE 'threshold.h'
C
      INTEGER NGPB
C
      INTEGER IGP
C
      REAL RHOALP(MAXGPB),RHOBET(MAXGPB),RHOTOT(MAXGPB)
      REAL DRHOALP(MAXGPB),DRHOBET(MAXGPB),DRHOAB(MAXGPB)
      REAL DRHOTOT(MAXGPB),KINALP(MAXGPB),KINBET(MAXGPB),ZSPIN(MAXGPB)
C
      REAL VXALP(MAXGPB),VXBET(MAXGPB),VXGALP(MAXGPB),VXGBET(MAXGPB)
      REAL VXTALP(MAXGPB),VXTBET(MAXGPB),VCALP(MAXGPB),VCBET(MAXGPB)
      REAL VCGALP(MAXGPB),VCGBET(MAXGPB),VCG2ALP(MAXGPB),VCG2BET(MAXGPB)
      REAL VCTALP(MAXGPB),VCTBET(MAXGPB)
C
C     ------------------------------------------------------------------
C
C     *** Calculate exchange potential contribution ***
C
      IF (XTYP(1:6).EQ.'XALPHA') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXALPHA(RHOALP(IGP),VXALP(IGP))
            CALL VXALPHA(RHOBET(IGP),VXBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'LSD') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXLSD(RHOALP(IGP),VXALP(IGP))
            CALL VXLSD(RHOBET(IGP),VXBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PW86') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXPW86(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXPW86(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'B88') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXB88(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXB88(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PW91') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXPW91(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXPW91(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'ECMV') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXECMV(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXECMV(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'EV93') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXEV93(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXEV93(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'LB94') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXLB94(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXLB94(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'PBE96') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXPBE96(RHOALP(IGP),DRHOALP(IGP),
     $                   VXALP(IGP),VXGALP(IGP))
            CALL VXPBE96(RHOBET(IGP),DRHOBET(IGP),
     $                   VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'PBE98') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXPBE98(RHOALP(IGP),DRHOALP(IGP),
     $                   VXALP(IGP),VXGALP(IGP))
            CALL VXPBE98(RHOBET(IGP),DRHOBET(IGP),
     $                   VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'PBE99') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXPBE99(RHOALP(IGP),DRHOALP(IGP),
     $                   VXALP(IGP),VXGALP(IGP))
            CALL VXPBE99(RHOBET(IGP),DRHOBET(IGP),
     $                   VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'PBESOL') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXPBESOL(RHOALP(IGP),DRHOALP(IGP),
     $                    VXALP(IGP),VXGALP(IGP))
            CALL VXPBESOL(RHOBET(IGP),DRHOBET(IGP),
     $                    VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'OPTX') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXHC(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXHC(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'KT1X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXKT1(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXKT1(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'KT2X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXKT2(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXKT2(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'KT3X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXKT3(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXKT3(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'VMT09') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXVMT(RHOALP(IGP),DRHOALP(IGP),
     $                 VXALP(IGP),VXGALP(IGP))
            CALL VXVMT(RHOBET(IGP),DRHOBET(IGP),
     $                 VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'VMTSOL') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXVMTSOL(RHOALP(IGP),DRHOALP(IGP),
     $                    VXALP(IGP),VXGALP(IGP))
            CALL VXVMTSOL(RHOBET(IGP),DRHOBET(IGP),
     $                    VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'SO11') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXSO11(RHOALP(IGP),DRHOALP(IGP),
     $                 VXALP(IGP),VXGALP(IGP))
            CALL VXSO11(RHOBET(IGP),DRHOBET(IGP),
     $                 VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'N12') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXN12(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXN12(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'CAP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXCAP(RHOALP(IGP),DRHOALP(IGP),
     $                 VXALP(IGP),VXGALP(IGP))
            CALL VXCAP(RHOBET(IGP),DRHOBET(IGP),
     $                 VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'XNCAP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXNCAP(RHOALP(IGP),DRHOALP(IGP),
     $                    VXALP(IGP),VXGALP(IGP))
            CALL VXNCAP(RHOBET(IGP),DRHOBET(IGP),
     $                    VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'GAM') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXGAM(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXGAM(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'VS98') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXVS98(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                 VXALP(IGP),VXGALP(IGP),VXTALP(IGP),1)
            CALL VXVS98(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                 VXBET(IGP),VXGBET(IGP),VXTBET(IGP),1)
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PKZB') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXPKZB(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                  VXALP(IGP),VXGALP(IGP),VXTALP(IGP))
            CALL VXPKZB(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                  VXBET(IGP),VXGBET(IGP),VXTBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'TPSS') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXTPSS(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                  VXALP(IGP),VXGALP(IGP),VXTALP(IGP))
            CALL VXTPSS(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                  VXBET(IGP),VXGBET(IGP),VXTBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'M06L') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXM06(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                 VXALP(IGP),VXGALP(IGP),VXTALP(IGP),1)
            CALL VXM06(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                 VXBET(IGP),VXGBET(IGP),VXTBET(IGP),1)
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'M11') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXM11(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                 VXALP(IGP),VXGALP(IGP),VXTALP(IGP))
            CALL VXM11(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                 VXBET(IGP),VXGBET(IGP),VXTBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'MN12') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXMN12(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                  VXALP(IGP),VXGALP(IGP),VXTALP(IGP))
            CALL VXMN12(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                  VXBET(IGP),VXGBET(IGP),VXTBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'SCAN') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).LE.RHOTHRESH) CYCLE
          CALL VXSCAN(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                VXALP(IGP),VXGALP(IGP),VXTALP(IGP))
          CALL VXSCAN(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                VXBET(IGP),VXGBET(IGP),VXTBET(IGP))
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'B3LYP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXB3LYP(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),
     $                   VXGALP(IGP))
            CALL VXB3LYP(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),
     $                   VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:4).EQ.'PBE0') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXPBE0(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),VXGALP(IGP))
            CALL VXPBE0(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'M062X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXM06(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                 VXALP(IGP),VXGALP(IGP),VXTALP(IGP),4)
            CALL VXM06(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                 VXBET(IGP),VXGBET(IGP),VXTBET(IGP),4)
          END IF
        END DO
C
      ELSE IF (XTYP(1:5).EQ.'M06HF') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXM06(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                 VXALP(IGP),VXGALP(IGP),VXTALP(IGP),2)
            CALL VXM06(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                 VXBET(IGP),VXGBET(IGP),VXTBET(IGP),2)
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'M06') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXM06(RHOALP(IGP),DRHOALP(IGP),KINALP(IGP),
     $                 VXALP(IGP),VXGALP(IGP),VXTALP(IGP),3)
            CALL VXM06(RHOBET(IGP),DRHOBET(IGP),KINBET(IGP),
     $                 VXBET(IGP),VXGBET(IGP),VXTBET(IGP),3)
          END IF
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'CAMB88') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXCAMB88(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),
     $                    VXGALP(IGP))
            CALL VXCAMB88(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),
     $                    VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:7).EQ.'RCAMB88') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXRCAMB88(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),
     $                     VXGALP(IGP))
            CALL VXRCAMB88(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),
     $                     VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:6).EQ.'CAMPBE') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXCAMPBE(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),
     $                    VXGALP(IGP))
            CALL VXCAMPBE(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),
     $                    VXGBET(IGP))
          END IF
        END DO
C
      ELSE IF (XTYP(1:3).EQ.'HSE') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VXHSE(RHOALP(IGP),DRHOALP(IGP),VXALP(IGP),
     $                 VXGALP(IGP))
            CALL VXHSE(RHOBET(IGP),DRHOBET(IGP),VXBET(IGP),
     $                 VXGBET(IGP))
          END IF
        END DO

C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** Calculate correlation potential contribution ***
C
      IF (CTYP(1:4).EQ.'PVWN') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCPVWN(RHOTOT(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'RVWN') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCRVWN(RHOTOT(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'PW92') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCPW92(RHOTOT(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'P86') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCP86(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP),
     $                 VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                 VCG2ALP(IGP),VCG2BET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'LYP') THEN
C
        DO IGP=1,NGPB
          IF ((RHOALP(IGP).GT.GPTHRESH(IGP)).AND.
     $        (RHOBET(IGP).GT.GPTHRESH(IGP))) THEN
            CALL VCLYP(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                 DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP),
     $                 VCALP(IGP),VCGALP(IGP),VCG2ALP(IGP))
            CALL VCLYP(RHOBET(IGP),RHOALP(IGP),RHOTOT(IGP),
     $                 DRHOBET(IGP),DRHOALP(IGP),DRHOTOT(IGP),
     $                 VCBET(IGP),VCGBET(IGP),VCG2BET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'PW91') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCPW91(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP),
     $                  VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                  VCG2ALP(IGP),VCG2BET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:6).EQ.'PBESOL') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCPBESOL(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP),
     $                    VCALP(IGP),VCBET(IGP),VCGALP(IGP),
     $                    VCGBET(IGP),VCG2ALP(IGP),VCG2BET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'SPBE') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCSPBE(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP),
     $                  VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                  VCG2ALP(IGP),VCG2BET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'PBE') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCPBE(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP),
     $                 VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                 VCG2ALP(IGP),VCG2BET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'KT2C') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCKT2(RHOTOT(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'KT3C') THEN
C
        DO IGP=1,NGPB
          IF ((RHOALP(IGP).GT.GPTHRESH(IGP)).AND.
     $        (RHOBET(IGP).GT.GPTHRESH(IGP))) THEN
            CALL VCKT3(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                 DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP),
     $                 VCALP(IGP),VCGALP(IGP),VCG2ALP(IGP))
            CALL VCKT3(RHOBET(IGP),RHOALP(IGP),RHOTOT(IGP),
     $                 DRHOBET(IGP),DRHOALP(IGP),DRHOTOT(IGP),
     $                 VCBET(IGP),VCGBET(IGP),VCG2BET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'SO11') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCSO11(RHOTOT(IGP),DRHOTOT(IGP),ZSPIN(IGP),VCALP(IGP),
     $                  VCBET(IGP),VCGALP(IGP),VCGBET(IGP),VCG2ALP(IGP),
     $                  VCG2BET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'N12') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCN12(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),DRHOALP(IGP),
     $                 DRHOBET(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP),
     $                 VCGALP(IGP),VCGBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'GAM') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCGAM(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),DRHOALP(IGP),
     $                 DRHOBET(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP),
     $                 VCGALP(IGP),VCGBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'VS98') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCVS98(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                  DRHOALP(IGP),DRHOBET(IGP),KINALP(IGP),
     $                  KINBET(IGP),ZSPIN(IGP),VCALP(IGP),
     $                  VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                  VCTALP(IGP),VCTBET(IGP),1)
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'PKZB') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCPKZB(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                  DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP),
     $                  KINALP(IGP),KINBET(IGP),ZSPIN(IGP),
     $                  VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                  VCG2ALP(IGP),VCG2BET(IGP),VCTALP(IGP),
     $                  VCTBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'TPSS') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCTPSS(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                  DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP),
     $                  DRHOAB(IGP),KINALP(IGP),KINBET(IGP),ZSPIN(IGP),
     $                  VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                  VCG2ALP(IGP),VCG2BET(IGP),VCTALP(IGP),
     $                  VCTBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'M06L') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCM06(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),DRHOALP(IGP),
     $                 DRHOBET(IGP),KINALP(IGP),KINBET(IGP),ZSPIN(IGP),
     $                 VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                 VCTALP(IGP),VCTBET(IGP),1)
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'M11') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCM11(RHOTOT(IGP),DRHOTOT(IGP),KINALP(IGP),KINBET(IGP),
     $                 ZSPIN(IGP),VCALP(IGP),VCBET(IGP),VCGALP(IGP),
     $                 VCGBET(IGP),VCG2ALP(IGP),VCG2BET(IGP),
     $                 VCTALP(IGP),VCTBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'MN12') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCMN12(RHOTOT(IGP),DRHOTOT(IGP),KINALP(IGP),
     $                  KINBET(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP),
     $                  VCGALP(IGP),VCGBET(IGP),VCG2ALP(IGP),
     $                  VCG2BET(IGP),VCTALP(IGP),VCTBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:4).EQ.'SCAN') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCSCAN(RHOTOT(IGP),DRHOTOT(IGP),KINALP(IGP),
     $                  KINBET(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP),
     $                  VCGALP(IGP),VCGBET(IGP),VCG2ALP(IGP),
     $                  VCG2BET(IGP),VCTALP(IGP),VCTBET(IGP))
          END IF
        END DO
C
      ELSE IF (CTYP(1:6).EQ.'RB3LYP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCRVWN(RHOTOT(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP))
            IF ((RHOALP(IGP).GT.GPTHRESH(IGP)).AND.
     $          (RHOBET(IGP).GT.GPTHRESH(IGP))) THEN
              CALL VCB3LYP(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                     DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP),
     $                     VCALP(IGP),VCGALP(IGP),VCG2ALP(IGP))
              CALL VCB3LYP(RHOBET(IGP),RHOALP(IGP),RHOTOT(IGP),
     $                     DRHOBET(IGP),DRHOALP(IGP),DRHOTOT(IGP),
     $                     VCBET(IGP),VCGBET(IGP),VCG2BET(IGP))
            END IF
          END IF
        END DO
C
      ELSE IF (CTYP(1:6).EQ.'PB3LYP') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCPVWN(RHOTOT(IGP),ZSPIN(IGP),VCALP(IGP),VCBET(IGP))
            IF ((RHOALP(IGP).GT.GPTHRESH(IGP)).AND.
     $          (RHOBET(IGP).GT.GPTHRESH(IGP))) THEN
              CALL VCB3LYP(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),
     $                     DRHOALP(IGP),DRHOBET(IGP),DRHOTOT(IGP),
     $                     VCALP(IGP),VCGALP(IGP),VCG2ALP(IGP))
              CALL VCB3LYP(RHOBET(IGP),RHOALP(IGP),RHOTOT(IGP),
     $                     DRHOBET(IGP),DRHOALP(IGP),DRHOTOT(IGP),
     $                     VCBET(IGP),VCGBET(IGP),VCG2BET(IGP))
            END IF
          END IF
        END DO
C
      ELSE IF (CTYP(1:5).EQ.'M062X') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCM06(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),DRHOALP(IGP),
     $                 DRHOBET(IGP),KINALP(IGP),KINBET(IGP),ZSPIN(IGP),
     $                 VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                 VCTALP(IGP),VCTBET(IGP),4)
          END IF
        END DO
C
      ELSE IF (CTYP(1:5).EQ.'M06HF') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCM06(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),DRHOALP(IGP),
     $                 DRHOBET(IGP),KINALP(IGP),KINBET(IGP),ZSPIN(IGP),
     $                 VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                 VCTALP(IGP),VCTBET(IGP),2)
          END IF
        END DO
C
      ELSE IF (CTYP(1:3).EQ.'M06') THEN
C
        DO IGP=1,NGPB
          IF (RHOTOT(IGP).GT.RHOTHRESH) THEN
            CALL VCM06(RHOALP(IGP),RHOBET(IGP),RHOTOT(IGP),DRHOALP(IGP),
     $                 DRHOBET(IGP),KINALP(IGP),KINBET(IGP),ZSPIN(IGP),
     $                 VCALP(IGP),VCBET(IGP),VCGALP(IGP),VCGBET(IGP),
     $                 VCTALP(IGP),VCTBET(IGP),3)
          END IF
        END DO
C
      END IF
C
C     ------------------------------------------------------------------
C
C     *** End of SUBROUTINE XCPOT ***
C
      END
